function tmaModule() {

	/**
	 * name of the modul. MUST be set in module
	 */
	this.name = "";

	/**
	 * properties of this module
	 * css: string for a single file or array for multiple files
	 * languages: array of supported languages (language-xx.txt)
	 */
	this.manifest = {};

	this.i18 = {};

	this.inititialised = false;

	/**
	 * stores the parent methods for inherited childs (modules)
	 * @see this.parent()
	 */
	var parent = this;

	/**
	 * contains redirection methods to parent
	 */
	var _parent = {};

	var settings = null;

	/**
	 * returns this object as parent to inherited childrens (moduls),
	 * so they can call parented methods <br>
	 * usage: this.parent().setup();
	 *
	 * @returns {Object} with parent methods called in childrens scope
	 */
	this.parent = function() {

		var self = this;

		if ($.isEmptyObject(_parent)) {
			$.each(parent, function(name, prop) {
				if (typeof prop == 'function') {
					_parent[name] = function() {
						return parent[name].apply(self, arguments);
					};
				}
			});
		}

		return _parent;

	};

	this.path = '';

	/**
	 * modules setup, loads language
	 *
	 * @returns Promise
	 */
	this.setup = function() {

		return this.loadLanguage();

	};

	/**
	 * load modules language file
	 *
	 * @returns Promise
	 */
	this.loadLanguage = function() {

		var self = this;

		var deferred = $.Deferred();
		var pipe = deferred;

		// Get the "language-??.txt" file

		/*var lng = '';
		if ( $.inArray( tma.i18.language_code, this.manifest.languages ) >= 0 ) {
			// load localized language
			lng = tma.i18.language_code;
		} else if ( $.inArray( 'en', this.manifest.languages ) >= 0 ) {
			// load default language
			lng = 'en'; // TODO customize default language?
		}*/

		// set language defaults, if not existing
		self.i18 = {
				module: {
					title: self.name
				}
		};

		//if (lng) {
		if (!this.manifest.nolngfile) {

			tma.progressMax++;
			pipe = pipe.then(function() {

				return tma.apiGetLanguage(self.name).done(function(lang) {

					$.extend( true, self.i18, lang);

				});

			});

		}

		deferred.resolve();

		return pipe.promise();

	};

	/**
	 * modules initialisation, loads settings and dependencies (css, js) described in the manifest
	 *
	 * @returns Promise
	 */
	this.init = function() {

		var self = this;

		var deferred = $.Deferred();
		var pipe = deferred;

		if (!this.inititialised) {

			// load module settings
			tma.progressMax++;
			pipe = pipe.then(function() {

				return tma.apiListSettings(self.name).done(function(data) {
					settings = data.setting;
				});

			});

			// Get the JavaScript file(s)

			if (typeof this.manifest.js == 'string') {

				// add css
				tma.progressMax++;
				pipe = pipe.then(function() {

					return tma.fn.loadJs(self.path + self.manifest.js);

				});

			} else if (typeof this.manifest.js == 'object') {

				$.each(this.manifest.js, function(i, js) {

					tma.progressMax++;
					pipe = pipe.then(function() {

						return tma.fn.loadJs(self.path + js);

					});

				});

			}

			// Get the css file(s)

			if (typeof this.manifest.css == 'string') {

				// add css
				tma.progressMax++;
				pipe = pipe.then(function() {

					return tma.fn.loadCss(self.path + self.manifest.css);

				});

			} else if (typeof this.manifest.css == 'object') {

				$.each(this.manifest.css, function(i, css) {

					tma.progressMax++;
					pipe = pipe.then(function() {

						return tma.fn.loadCss(self.path + css);

					});

				});

			}

		}
		this.inititialised = true;

		deferred.resolve();

		return pipe.promise();

	};

	/**
	 * save a modules config value
	 *
	 * @param key		name of the config param
	 * @param value
	 * @returns any value
	 */
	this.setSetting = function (key, value) {

		var self = this;

		settings[key] = value;

		var params = {
				'function':	'setting',
				action:		'set!',
				key:		key,
				value:		value,
				module:		this.name
		};

		return $.getJSON('/api', params).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in " + self.name + ".setSetting()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					key: key ,
					value: value,
				}
			});

		});

	};

	/**
	 * load a modules config value
	 *
	 * @param key		name of the config param
	 * @param _default	default value, if setting doesnt exist
	 * @returns any value
	 */
	this.setting = function (key, _default) {

		if ( (!settings) || (settings[key] == undefined) ) {
			return _default;
		}

		return settings[key];

	};

	/**
	 * load a local modules config value
	 *
	 * @param key		name of the config param
	 * @param _default	default value, if setting doesnt exist
	 * @returns any value
	 */
	this.localSetting = function (key, _default) {

		/*if ( $.jStorage.currentBackend() ) {
			return $.jStorage.get(key, _default);
		}*/

		return amplify.store( this.name + '_' + key ) || _default;

	};

	/**
	 * save a local modules config value
	 *
	 * @param key		name of the config param
	 * @param value
	 */
	this.setLocalSetting = function (key, value) {

		/*if ( $.jStorage.currentBackend() ) {
			$.jStorage.set(key, value);
		}*/

		amplify.store( this.name + '_' + key, value );

	};

}
