/**
 * channels module
 */
tma.registerModule( channelspage );
channelspage.prototype = new tmaModule();
function channelspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'channelspage';
	
	this.pageGroup = 'channels';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.parent().setup();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {
		
		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.addPage( this, this.i18.menu_caption );
			
		}

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#content').empty();
			$("#timestamp").text(new Date(tma.channelsObj.timestamp).toLocaleString() );

			var $frame = $('<div/>', {
				'class': "channels"
			}).appendTo('DIV#content');

			$('<h3 class="favGroup" />').appendTo($frame);

			// show channels list

			self.showChannels();

			// add event listener to select channel
			$frame.on('click', 'DIV.channel', function() {
				// 'this' is the clicked element
				self.selectCurrentChannel( $(this) );
			});
			
			/** @returns MenuManager */
			var menu;
			if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

				menu.insertItemAfter(
						self.name,
						self,
						'channels-config',
						'<i class="fa fa-gear" />',
						self.toggleConfig
				).addClass('config');
				
			}

			self.insertConfig();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// hide and clear config frame, if exists
		$('DIV#config').hide().empty().removeClass('channels-config');
		
		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.removeItem('channels-config');
			
		}

	};

	/**
	 * (de)activate config
	 */
	this.toggleConfig = function() {

		$('DIV#config').slideToggle();

	};

	/**
	 * selects a channel and marks the element with css class 'current-channel'
	 *
	 * @param {jQueryObject} $channel	channel element with attribute channelid
	 */
	this.selectCurrentChannel = function ($channel) {

		var channel;
		if ( channel = tma.channel( $channel.attr('channelid') ) ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).done(function() {
				// set new current channel mark
				$('.current-channel').removeClass('current-channel');
				$channel.addClass('current-channel');
			}).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.dialog_title_error,
						text:	self.i18.error_channel_set
					}).confirm();
				}

			});
		}

	};

	/**
	 * creates a channel div
	 *
	 * @param channel		channel object
	 * @param displayMode	1: display logo; 2: display name; 3: display both
	 * @returns {jQueryObject}
	 */
	this.createChannelPanel = function (channel, displayMode) {

		var $channel = $('<div/>', {
			'class': 'channel',
			channelid: channel.logoid
		});

		if ( displayMode & 1 ) {

			$logo = $('<div class="channellogo">').appendTo($channel);
			if (channel.logo) {
				$logo.addClass('logo-' + channel.logo.logoid);
			} else if (displayMode == 1) {
				$logo.text(channel.servicename);
			}

		}

		if ( displayMode & 2 ) {

			$text = $('<span>' + channel.servicename + '</span>').appendTo($channel);

		}

		return $channel;

	};

	this.showChannels = function () {

		var favGroup = tma.setting('defaultfavourites', '');
		var displayMode = parseInt( tma.setting('channel_display', 3) );

		$('DIV.channels DIV.channel').remove();

		if (!favGroup) {
			$('DIV.channels .favGroup').text('');
		} else if (favGroup == '__tv') {
			$('DIV.channels .favGroup').text('TV');
		} else if (favGroup == '__radio') {
			$('DIV.channels .favGroup').text('Radio');
		} else {
			$('DIV.channels .favGroup').text(favGroup);
		}

		var channels = tma.getChannels(favGroup);

		$.each(channels, function(nr, channel) {

			self.createChannelPanel(channel, displayMode).appendTo('DIV.channels');

		});

		// get current channel and current recording activity and mark
		tma.apiGetStatus().done(function(status) {

			$('DIV.channel[channelid="' + status.current_logoid + '"]')
				.addClass('current-channel');

			$.each(status.recording, function(nr, recording) {

				if (recording.rectype) {

					$('DIV.channel[channelid="' + recording.logoid + '"]')
						.addClass('rec-channel');

				}

			});


		});

		$("#counter").text( tma.i18.record_counter.format( channels.length ) );

		// apply the channelsacross setting
		var ow = parseInt($('DIV.channel').outerWidth(true)) + 1;
		var w = ow * parseInt(this.setting('channelsacross', 10));
		$('DIV.channels').css( { maxWidth: w } );

	};

	this.insertConfig = function () {

		var template = (
			'<div class="setting cfg-columns">' +
				'<span class="label">{config.columns}</span>' +
				'<span class="value"></span> ' +
				'<button class="lessmore less" title="{config.less_cols}">&#x25C0;</button>' +
				'<button class="lessmore more" title="{config.more_cols}">&#x25B6;</button>' +
			'</div>' +
			'<div class="setting favourites">' +
				'<span class="label">{config.fav}</span>' +
				'<select name="favourites" size="1">' +
				'<option value="">{config.fav_all}</option>' +
				'<option value="__tv">{config.fav_tv}</option>' +
				'<option value="__radio">{config.fav_radio}</option>' +
				'</select>' +
			'</div>' +
			'<div class="setting logotext">' +
				'<span class="label">{config.display_channel}</span>' +
				'<input type="radio" name="logotext" value="1"><label>{config.display_logo}</label> ' +
				'<input type="radio" name="logotext" value="2"><label>{config.display_name}</label> ' +
				'<input type="radio" name="logotext" value="3"><label>{config.display_both}</label> ' +
			'</div>'
		).format(this.i18);

		$('DIV#config')
		.append(template)
		.addClass('channels-config');

		// set channelsacross setting
		$('DIV#config DIV.cfg-columns *.value').text(
				this.setting('channelsacross', 10)
		);
		
		// change channelsacross setting
		$('DIV#config DIV.cfg-columns *.less').click(function() {

			var $frame = $('DIV.channels');
			var ow = parseInt($('DIV.channel').outerWidth(true)) + 1;
			var columns = parseInt(self.setting('channelsacross', 10)) - 1;
			if (columns > 0) {
				$frame.css('max-width', columns * ow);
				self.setSetting('channelsacross', columns);
				$('DIV#config DIV.cfg-columns *.value').text(columns);
			}

		});
		$('DIV#config DIV.cfg-columns *.more').click(function() {

			var ow = parseInt($('DIV.channel').outerWidth(true)) + 1;
			var columns = parseInt(self.setting('channelsacross', 10)) + 1;
			var w = columns * ow;
			if ( w < $('#content').width() ) {
				$('DIV.channels').css('max-width', w);
				self.setSetting('channelsacross', columns);
				$('DIV#config DIV.cfg-columns *.value').text(columns);
			}

		});

		var $fav = $('DIV#config SELECT[name="favourites"]');
		$.each(tma.favourites.groups, function(nr, group) {

			if (group.count !=0) {
				//Only list groups with members
				$fav.append(
						'<option value="' + group.name + '">' +
						group.name + " (" + group.count + ")" +
						'</option>'
				);
			}

		});
		$fav.val( tma.setting('defaultfavourites', '') );
		$fav.change(function() {

			tma.setSetting( 'defaultfavourites', $(this).val() );
			self.showChannels();

		});

		$('DIV#config INPUT[name="logotext"]')
			.click(function() {

				tma.setSetting( 'channel_display', $('DIV#config INPUT[name="logotext"]:checked').val() );
				self.showChannels();

			})
			.filter('[value="' +  tma.setting('channel_display', 3) + '"]' ).attr('checked', true);

	};

}
