/**
 * control module
 */
tma.registerModule( configpage );
configpage.prototype = new tmaModule();
function configpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'configpage';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		// set font size from setting
		var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );
		$('BODY').css('fontSize', fs + 'px');

		return this.parent().setup();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.addPage( this, this.i18.menu_caption );
			
		}

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			self.insertConfig();

			$('#config').show();
			$('#timestamp').hide();
			$('#counter').hide();

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		$('#config').removeClass('globals-config');
		$('#timestamp').show();
		$('#counter').show();

	};

	this.insertConfig = function () {

		var template =
			'<div class="setting language-mode">' +
				'<span class="label">' + this.i18.language_mode + '</span>' +
				'<select name="language-mode" size="1">' +
					'<option value="pvr">' + this.i18.language_mode_pvr + ' (' + tma.config.language_code + ')</option>' +
				'</select>' +
				'<button class="button-save jk-but iconset ic-save"	title="' + self.i18.save_language + '"></button>' +
			'</div>' +
			'<div class="setting fontsize">' +
				'<span class="label">' + this.i18.font_size + '</span>' +
				'<button class="lessmore less" title="' + this.i18.dec_font_size + '">&#x25C0;</button>' +
				'<button class="lessmore more" title="' + this.i18.inc_font_size + '">&#x25B6;</button>' +
			'</div>' +
			'<div class="setting logos-aspect-ratio">' +
				'<span class="label">' + this.i18.logo_aspect_ratio + '</span>' +
				'<select name="logo-aspect-ratio" size="1">' +
					'<option value="54">54x34</option>' +
					'<option value="60">60x34</option>' +
					'<option value="78">78x34</option>' +
				'</select>' +
			'</div>' +
			'<div class="setting modules">' +
				'<span class="label">' + this.i18.modules + '</span>' +
				'<div class="modules-config-frame">' +
					'<div class="modules-frame used-modules">' +
						'<h1>' + this.i18.selected_modules + '</h1>' +
						'<div class="modules-list">' +
						'</div>' +
					'</div>' +
					'<div class="module-buttons">' +
						'<button class="jk-but iconset ic-right"	disabled title="' + self.i18.remove_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-up"		disabled title="' + self.i18.up_tooltip		+ '"></button>' +
						'<button class="jk-but iconset ic-down"		disabled title="' + self.i18.down_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-left"		disabled title="' + self.i18.add_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-save"				 title="' + self.i18.save_modules	+ '"></button>' +
						'<button class="jk-but iconset ic-home"		disabled title="' + self.i18.home_tooltip	+ '"></button>' +
						'<button class="jk-but iconset ic-reset"			 title="' + self.i18.reset_modules	+ '"></button>' +
					'</div>' +
					'<div class="modules-frame available-modules">' +
						'<h1>' + this.i18.available_modules + '</h1>' +
						'<div class="modules-list">' +
						'</div>' +
					'</div>' +
				'</div>' +
			'</div>';

		$('DIV#config')
		.append(template)
		.addClass('globals-config');

		/* modules config (device - all modules) */

		var moduleTemplate =
			'<div class="module jk-list-row" module="{name}" data-type="{moduletype}">' +
				'<input type="checkbox">' +
				'<span class="module-name">{caption}</span>' +
				'<button class="jk-but iconset ic-info" title="' + this.i18.info_tooltip + '"></button>' +
			'</div>';

		// fill "available modules"
		var availableModules = tma.fn.getModules();
		$.each(availableModules, function( i, name ) {

			var moduleData = tma.fn.getModuleData(name);
			if (moduleData.manifest) {

				var $md = $(
						moduleTemplate.format({
							name:		moduleData.name,
							id:			'a-' + moduleData.name,
							caption:	moduleData.name,
						})
				);

				$md.appendTo('DIV#config DIV.available-modules DIV.modules-list');

				if ( moduleData.isInitialised() ) {
					// module is in use
					$md.hide();
					$md.find('SPAN').text(moduleData.module.i18.module.title);
					$md.attr('data-type', moduleData.manifest.type);
					if (moduleData.module.i18.module.info) $md.find('BUTTON.ic-info').show();
				} else if ( moduleData.isCreated() && moduleData.isLanguageLoaded() ) {
					// module is not in use, but created and language is available
					$md.find('SPAN').text(moduleData.module.i18.module.title);
					$md.attr('data-type', moduleData.manifest.type);
					if (moduleData.module.i18.module.info) $md.find('BUTTON.ic-info').show();
				} else if ( moduleData.isCreated() ) {
					// module is not in use, but created and language is not available
					moduleData.module.loadLanguage().done(function() {
						$md.find('SPAN').text(moduleData.module.i18.module.title);
						$md.attr('data-type', moduleData.manifest.type);
						if (moduleData.module.i18.module.info) $md.find('BUTTON.ic-info').show();
					});
				} else {
					// module is not in use and not created
					moduleData.loadManifest().done(function() {
						tma.fn.loadJs( moduleData.path() + moduleData.name + '.js' );
						$(moduleData).on('tmaModuleCreated', function(e, module) {
							module.loadLanguage().done(function() {
								$md.find('SPAN').text(module.i18.module.title);
								$md.attr('data-type', moduleData.manifest.type);
								if (module.i18.module.info) $md.find('BUTTON.ic-info').show();
							});
						});
					});
				}

			}

		});

		// fill "used modules"
		var usedModules = tma.modulesToLoad();
		$.each(usedModules, function( i, name ) {

			var moduleData = tma.fn.getModuleData(name);
			if (moduleData) {

				var $md = $(
						moduleTemplate.format({
							name:		moduleData.name,
							id:			'u-' + moduleData.name,
							caption:	moduleData.module.i18.module.title,
							moduletype:	moduleData.manifest.type,
						})
				);
				if (moduleData.module.i18.module.info) $md.find('BUTTON.ic-info').show();
				$md.appendTo('DIV#config DIV.used-modules DIV.modules-list');

			}

		});

		// set current startpage
		var startpage = tma.localSetting('startpage', 'none');
		$('DIV#config DIV.used-modules DIV.modules-list DIV.module[module="' + startpage + '"]').addClass('start-module');

		$('DIV#config DIV.modules-list').on('click', 'DIV.module', function(e) {

			// click on div.module works like a <select> list with multiple selections
			if (!e.metaKey && !e.ctrlKey) {

				$('DIV#config DIV.modules-list DIV.module-selected')
				.removeClass('module-selected')
				.find('INPUT').prop('checked', false);

			}

			$(this).find('INPUT').click();

		});

		$('DIV#config DIV.used-modules DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			// click on used module deselects all available modules
			$('DIV#config DIV.available-modules DIV.module-selected').removeClass('module-selected');
			$('DIV#config DIV.available-modules INPUT:checked').prop('checked', false);

		});

		$('DIV#config DIV.available-modules DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			// click on available module deselects all used modules
			$('DIV#config DIV.used-modules DIV.module-selected').removeClass('module-selected');
			$('DIV#config DIV.used-modules INPUT:checked').prop('checked', false);

		});

		$('DIV#config DIV.modules-list').on('click', 'DIV.module INPUT', function(e) {

			e.stopPropagation();

			// a module was (de-)selected
			$(this).parents('DIV.module').toggleClass('module-selected');

			var isUsedModule = $(this).parents('DIV.modules-frame').hasClass('used-modules');
			var $sm = $(this).parents('DIV.modules-list').find('DIV.module-selected');

			// set buttons visibility
			var $buttons = $('DIV#config DIV.module-buttons');
			$buttons.find('BUTTON.ic-right'	).attr( 'disabled', ($sm.length == 0) || !isUsedModule );
			$buttons.find('BUTTON.ic-left'	).attr( 'disabled', ($sm.length == 0) || isUsedModule );
			$buttons.find('BUTTON.ic-up'	).attr( 'disabled', ($sm.length == 0) || !isUsedModule || $sm.first().is(':first-child') );
			$buttons.find('BUTTON.ic-down'	).attr( 'disabled', ($sm.length == 0) || !isUsedModule || $sm.last().is(':last-child') );
			$buttons.find('BUTTON.ic-home'	).attr( 'disabled', ($sm.length != 1) || !isUsedModule || !$sm.first().is('[data-type="page"]') );

		});

		// remove button
		$('DIV#config DIV.module-buttons BUTTON.ic-right').click(function() {

			// get all selected modules in 'used modules'
			var $selected = $('DIV#config DIV.used-modules DIV.module-selected');

			// show them in 'available modules'
			$selected.each(function() {

				var name = $(this).attr('module');
				$('DIV#config DIV.available-modules DIV.module[module="' + name + '"]').show();

			});

			$selected.remove();

			// update button's state
			$(this).siblings('.ic-up').attr( 'disabled', true );
			$(this).siblings('.ic-down').attr( 'disabled', true );
			$(this).siblings('.ic-right').attr( 'disabled', true );
			$(this).siblings('.ic-left').attr( 'disabled', true );

		});

		// add button
		$('DIV#config DIV.module-buttons BUTTON.ic-left').click(function() {

			// get all selected modules in 'available modules'
			var $selected = $('DIV#config DIV.available-modules DIV.module-selected');

			// add them to 'used modules'
			$selected.each(function() {

				$('DIV#config DIV.used-modules DIV.modules-list').append( $(this ).clone());

			});

			$selected.removeClass('module-selected').hide().find('INPUT').attr('checked', false);

			// update button's state
			var $selected = $('DIV#config DIV.used-modules DIV.module-selected');
			$(this).siblings('.ic-up').attr( 'disabled', $selected.first().is( ':first-child' ) );
			$(this).siblings('.ic-down').attr( 'disabled', $selected.last().is( ':last-child' ) );
			$(this).siblings('.ic-right').attr( 'disabled', false );
			$(this).siblings('.ic-left').attr( 'disabled', true );


		});

		// up button
		$('DIV#config DIV.module-buttons BUTTON.ic-up').click(function() {

			// get all selected modules
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected');

			// move them one positiop up
			$selected.each(function() {

				$(this).prev().before( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.first().is( ':first-child' ) );
			$(this).siblings('.ic-down').attr( 'disabled', false );

		});

		// down button
		$('DIV#config DIV.module-buttons BUTTON.ic-down').click(function() {

			// get all selected modules
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected');

			// move them one positiop down
			$selected.each(function() {

				$(this).next().after( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.last().is( ':last-child' ) );
			$(this).siblings('.ic-up').attr( 'disabled', false );

		});

		// home button
		$('DIV#config DIV.module-buttons BUTTON.ic-home').click(function() {

			// get first selected page module
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module-selected[data-type="page"]').first();

			if ( $selected.length ) {

				if ( $selected.hasClass('start-module') ) {
					$selected.removeClass('start-module');
				} else {

					// remove previous home
					$('DIV#config DIV.used-modules DIV.modules-list DIV.start-module').removeClass('start-module');
					// set home
					$selected.addClass('start-module');

				}

			}

		});

		// save button
		$('DIV#config DIV.module-buttons BUTTON.ic-save').click(function() {

			// get all 'used modules'
			var $selected = $('DIV#config DIV.used-modules DIV.modules-list DIV.module');

			var list = new Array();

			// move them one positiop down
			$selected.each(function() {

				list.push( $(this).attr('module') );

			});

			tma.setLocalSetting( 'modules', list.join(',') );

			// start page
			var home = $('DIV#config DIV.used-modules DIV.modules-list DIV.start-module').attr('module');
			if (home) {
				tma.setLocalSetting( 'startpage', home );
			} else {
				tma.setLocalSetting( 'startpage', null );
			}

			window.location.reload();

		});

		// reset button
		$('DIV#config DIV.module-buttons BUTTON.ic-reset').click(function() {

			new jkqDialog({
				title: self.i18.reset_confirm_caption,
				text: self.i18.reset_confirm_text,
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'ok') {

					location.replace('reset-modules.html');

				}

			});
		});

		// info button
		$('DIV#config DIV.modules-list BUTTON.ic-info').click(function(e) {

			e.stopPropagation();

			var name = $(this).parents('DIV.module').attr('module');
			var moduleData = tma.fn.getModuleData(name);
			if (
					(moduleData) &&
					(moduleData.module.i18.module.info)
			) {

				self.getModuleDescription(moduleData).done(function(info) {

					var infoWindow = new jkqWindow();
					infoWindow.width = 450;

					infoWindow.showModal();
					infoWindow.setTitle( moduleData.module.i18.module.title );
					infoWindow.closeOnOutsideClick();

					var $content = infoWindow.container.find('DIV.jkqWindowContent');

					$content.append(
							'<div class="module-info">' +
							info +
							'</div>'
					);

					infoWindow.adjustPos();

				});

			}

		});

		/* logo aspect ratio setting (PVR - all modules) */

		if (tma.logos) {

			// change logos aspect ratio setting
			$('DIV#config SELECT[name="logo-aspect-ratio"]')
			.val( tma.setting('logodisplaywidth', 54) )
			.change(function() {

				tma.setSetting( 'logodisplaywidth', $(this).val() );
				tma.refreshLogoStyles();

			});

		} else {

			// if logos aren't available, hide logos aspect ratio setting
			$('DIV#config DIV.logos-aspect-ratio').hide();

		}

		/* font-size setting (device - all modules) */

		$('DIV#config DIV.fontsize .less').click(function() {

			var fs = parseInt( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			if (fs > 5) {
				fs--;
				$('BODY').css('fontSize', fs + 'px');
				tma.setLocalSetting('fontsize', fs);
			}

		});
		$('DIV#config DIV.fontsize .more').click(function() {

			var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			fs++;
			$('BODY').css('fontSize', fs + 'px');
			tma.setLocalSetting('fontsize', fs);

		});

		/* language mode setting (local - all modules) */

		// add languages
		if (this.i18.languages) {
			$.each(this.i18.languages, function(lng, title) {

				$('DIV#config SELECT[name="language-mode"]').append(
						'<option value="' + lng + '">' + title + ' (' + lng + ')</option>'
				);

			});
		}

		// save language button
		$('DIV#config DIV.language-mode .button-save').click(function() {

			tma.setLocalSetting( 'languagemode', $('DIV#config SELECT[name="language-mode"]').val() );

			window.location.reload();

		});

		// apply current setting
		$('DIV#config SELECT[name="language-mode"]')
		.val( tma.localSetting('languagemode', 'pvr') );

	};

	var descriptionsCache = {};

	/**
	 * reads the module description
	 *
	 * @param {tmaModuleData} moduleData
	 */
	this.getModuleDescription = function( moduleData ) {

		var deferred = $.Deferred();
		var pipe = deferred;

		if (
				(moduleData) &&
				(moduleData.module.i18.module.info)
		) {

			if ( moduleData.module.i18.module.info.match( /\.html$/ ) ) {

				// moduleData.module.i18.module.info is a filename
				var filename = moduleData.path() + 'lng/' + moduleData.module.i18.module.info;

				if (!descriptionsCache[moduleData.name]) {

					// load module description
					$.get(filename).done(function(data) {

						descriptionsCache[moduleData.name] = data;
						deferred.resolve(descriptionsCache[moduleData.name]);

					});

				} else {

					deferred.resolve(descriptionsCache[moduleData.name]);

				}

			} else {

				// moduleData.module.i18.module.info is a text string
				descriptionsCache[moduleData.name] = moduleData.module.i18.module.info;
				deferred.resolve(descriptionsCache[moduleData.name]);

			}

		} else {

			deferred.resolve('');

		}

		return pipe.promise();

	};

}
