/**
 * EPG module
 */
tma.registerModule( epgpage );
epgpage.prototype = new tmaModule();
function epgpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgpage';
	
	this.pageGroup = 'epg';

	var self = this;
	
	/**
	 * list of EPG-data per channel (index: channelid)
	 * 
	 * @type ChannelData[]
	 */
	var channelsData = {};

	/**
	 * Starttime of the epg-timespan in milliseconds rounded to minutes
	 * 
	 * @type {Number}
	 */
	this.starttime;

	/**
	 * Endtime of the epg-timespan in milliseconds rounded to minutes
	 * 
	 * @type {Number}
	 */
	this.endtime;
	
	/**
	 * default query length in minutes
	 */
	this.timeSliceLength = 360; // 6h => 4 slices per day
	
	/**
	 * total time range ( maxTimeSlices x timeSliceLength )
	 */
	this.maxTimeSlices = 28;	// 7 days
	
	/**
	 * time scale: Pixel per Minute
	 */
	this.pxPerMin = 7;
	
	this.channels;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.parent().setup();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.addPage( this, this.i18.menu_caption );
			
		}

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {
			
			$('<div id="debug">').appendTo('BODY')
			.css({
				position: 'absolute',
				top: 0,
				right: 0,
			});

			self.showEPG();

			/*this.parentMenu.addItem(
				this,
				'channels-config',
				'<i class="fa fa-gear" />',
				this.toggleConfig
			).addClass('config');

			this.insertConfig();*/

			$("#tma-status").hide(); // prevent vertical document scrollbar (vScroll in EPG-frame)

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// hide and clear config frame, if exists
		//$('DIV#config').hide().empty().removeClass('channels-config');

		//this.parentMenu.removeItem('channels-config');
		
		$('DIV#debug').remove();

		// remove mode-class
		$('DIV#content').removeClass('mode-edit-timer');

		// remove event handling
		$(tma).off('tmaTimerAdded.epgpage');
		$(tma).off('tmaTimerModified.epgpage');

		// show app status
		$("#tma-status").show();

	};

	/**
	 * (de)activate config
	 */
	this.toggleConfig = function() {

		$('DIV#config').slideToggle();

	};
	
	/**
	 * callback of timer modification drag-Objects
	 * @this {jkqDragDrop}
	 */
	this.onTimerModified = function(left, top, width) {

		/** @type jQueryObject */
		var $source = this.$source;
		var timer = $source.prop('timer');
		var start = Math.round( left		  / self.pxPerMin) * 60000 + self.starttime;
		var end   = Math.round((left + width) / self.pxPerMin) * 60000 + self.starttime;

		$source.find('SPAN.epg-mark-fn-start').text(tma.date('H:i', start));
		$source.find('SPAN.epg-mark-fn-end'  ).text(tma.date('H:i', end  ));

		if (
				( tma.date('H:i', start) != tma.date('H:i', timer.starttime) ) || 
				( tma.date('H:i', end  ) != tma.date('H:i', timer.endtime  ) )
		) {
			$source.addClass('epg-mark-modified');
		} else {
			$source.removeClass('epg-mark-modified');
		}

	};

	/**
	 * creates a reservation bar for a timer, if it is in the EPG range
	 *
	 * @param {jQuery}		$bar		parent timer bar
	 * @param {TmaTimer}	timer		tma-timer object
	 * @returns {jqObject} or false
	 */
	this.createTimerMarker = function($bar, timer) {
		
		/**
		 * Valid days for recurring reservation
		 * default: every day is valid
		 */ 
		var recurringValidDays = [1,1,1,1,1,1,1];
		
		if (timer.reservationtype == 2) {
			// weekend, only sunday and saturday is valid
			recurringValidDays = [1,0,0,0,0,0,1];
		} else if (timer.reservationtype == 4) {
			// weekdays, only monday to friday is valid
			recurringValidDays = [0,1,1,1,1,1,0];
		} 

		var timer_start = new Date(timer.starttime).getTime();
		var timer_end	= new Date(timer.endtime).getTime();
		
		var loop = 0;
		var day = 0;
		while (timer_start < this.endtime) {
			
			var dayOfWeek = parseInt(tma.date('w', timer_start));
			if (
					(recurringValidDays[dayOfWeek]) &&
					(
							((timer_start	>= this.starttime) && (timer_start	<= this.endtime)) ||
							((timer_end		>= this.starttime) && (timer_end	<= this.endtime)) ||
							((timer_start	<= this.starttime) && (timer_end	>= this.endtime))
					) 
			) {

				var pos = Math.round( (timer_start - this.starttime) * this.pxPerMin / 60000 );
				var w	= Math.round( (timer_end   - timer_start   ) * this.pxPerMin / 60000 );
				
				var timerNameTmpl = (
						'<span class="epg-mark-fn-name">{name}</span>'
				).format({
					name: timer.getName(),
				});
				
				var timerSpanTmpl =	(
						'<br>' +
						'<span class="epg-mark-fn-time">' +
						'<span class="epg-mark-fn-start">{starttime}</span>' +
						' - ' +
						'<span class="epg-mark-fn-end">{endtime}</span>' +
						'</span>'
				).format({
					starttime:	tma.date('H:i', timer.starttime),
					endtime:	tma.date('H:i', timer.endtime),
				});
				
				var timerButtonsTmpl = '';
				if (!loop) {
					// first occurance of the timer
					timerButtonsTmpl =	(
							'<button class="jk-but iconset ic-left epg-mark-fn-resize-left" title="{timer_edit_inline_tooltip}"></button>' +
							'<div class="epg-mark-fn-buttons">' +
							'{timername}<br>' +
							'<button class="jk-but iconset ic-clock" title="{timer_edit_tooltip}"></button>' +
							'<button class="jk-but iconset ic-save" title="{timer_save_tooltip}"></button>' +
							'<button class="jk-but iconset ic-delete" title="{timer_delete_tooltip}"></button>' +
							'<button class="jk-but iconset ic-reset" title="{timer_reset_tooltip}"></button>' +
							'{timerspan}' +
							'</div>' +
							'<button class="jk-but iconset ic-right epg-mark-fn-resize-right" title="{timer_edit_inline_tooltip}"></button>'
					).format($.extend({}, this.i18, {
						timername:	timerNameTmpl,
						timerspan:	timerSpanTmpl,
					}));
				} else {
					// recurring timer (not the first)
					timerButtonsTmpl =	(
							'<div class="epg-mark-fn-buttons">' +
							'{timername}<br>' +
							'<button class="jk-but iconset ic-left epg-mark-fn-jumptofirst" title="{timer_tofirst_tooltip}" day="{day}"></button>' +
							'<button class="jk-but iconset ic-clock" title="{timer_edit_tooltip}"></button>' +
							'{timerspan}' +
							'</div>'
					).format($.extend({}, this.i18, {
						timername:	timerNameTmpl,
						timerspan:	timerSpanTmpl,
						day:		day,
					}));
				}

				$marker = $(
						(
								'<div class="epg-mark epg-bg {typeclass}{typeext}" style="left:{left}px;width:{width}px;">' +
									'<div class="epg-mark-fn">{buttons}</div>' +
								'</div>'
						).format({
							typeclass:	timer.isrec ? 'epg-mark-rec' : 'epg-mark-play',
							typeext:	(loop) ? '-ff' : '',	
							left:		pos,
							width:		w,
							buttons:	timerButtonsTmpl,
						})
				)
				.attr('timerindex', timer.index)
				.prop('timer', timer)
				.click(function() {

					// edit reservation

					// check, if any module handles the 'editTimer' event
					if (tma.fn.editTimer) {
						var timer = $(this).prop('timer');
						tma.fn.editTimer( timer );
					}

				})

				.on('click', 'BUTTON.epg-mark-fn-jumptofirst', function(e) {
					
					// jump to the first occurance of this reservation
					e.stopPropagation();
					
					var day = parseInt($(this).attr('day'));
					
					var $container = $('DIV.epg-container');
					$container.scrollLeft($container.scrollLeft() - day * self.pxPerMin * 24*60 );					

				})

				.on('click', 'BUTTON.ic-save', function(e) {
					
					// edit modified reservation
					e.stopPropagation();

					// check, if any module handles the 'editTimer' event
					if (tma.fn.editTimer) {
						
						var $marker = $(this).parents('DIV.epg-mark');
						var timer = $marker.prop('timer');
						tma.fn.editTimer( timer ).done(function(editTimerInstance) {
							
							var left = $marker.position().left;
							var width = $marker.outerWidth();
							var start = Math.round( left		  / self.pxPerMin) * 60000 + self.starttime;
							var end   = Math.round((left + width) / self.pxPerMin) * 60000 + self.starttime;
							
							// start time
							$("#timerstart").val( tma.toPvrDate(start) );
							// end time
							$("#timerend").val( tma.toPvrDate(end) );

							editTimerInstance.refreshTimerEditTimes( /* calculate duration */ 2 ); 
							
						});
					}


				})

				.on('click', 'BUTTON.ic-reset', function(e) {
					
					// reset modified reservation
					e.stopPropagation();
					
					var $marker = $(this).parents('DIV.epg-mark');
					var timer = $marker.prop('timer');

					var timer_start = new Date(timer.starttime).getTime();
					var timer_end	= new Date(timer.endtime  ).getTime();

					$marker.css({
						left	: Math.round( (timer_start - self.starttime) * self.pxPerMin / 60000 ),
						width	: Math.round( (timer_end   - timer_start   ) * self.pxPerMin / 60000 )
					});

					$marker.find('SPAN.epg-mark-fn-start').text(tma.date('H:i', timer_start));
					$marker.find('SPAN.epg-mark-fn-end'  ).text(tma.date('H:i', timer_end  ));

					$marker.removeClass('epg-mark-modified');
					
				})

				.on('click', 'BUTTON.ic-delete', function(e) {

					e.stopPropagation();

					// delete reservation
					var timer = $(this).parents('DIV.epg-mark').prop('timer');

					var filename;
					if (tma.config.utftoppy || !timer.filename_hex) {
						filename = timer.filename;
					} else {
						filename = tma.decodeHexname(timer.filename_hex);
					}

					new jkqDialog({
						title: self.i18.timer_delete_confirm_caption,
						text:  self.i18.timer_delete_confirm_text.format(tma.removeExtension(filename)),
						buttons: {
							ok:				 tma.i18.respond_yes,
							jkqDialogCancel: tma.i18.respond_cancel
						}
					}).confirm(function(button) {

						if (button == 'ok') {

							tma.apiDeleteTimer(timer).done(function(delObject) {

								//if (delObject.mpg_status == 0) {
								$bar.find('DIV.epg-mark[timerindex="' + timer.index + '"]').remove();

								/*} else {

										new jkqAlert({
											title: tma.i18.dialog_title_error,
											text: 'status ' + delObject.mpg_status + ': delete file<br>"' + delObject.file + '"<br>failed!'
										}).confirm();

									}*/

							});

						}

					});

				})

				// drag marker start
				.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-left', function(e) {
					
					e.stopPropagation();
					
					self.timerResizeL.start(e, $(this).parents('.epg-mark'));
					self.timerResizeL.onDragMove = self.onTimerModified;
					
				})

				// drag marker end
				.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-right', function(e) {
					
					e.stopPropagation();
					
					self.timerResizeR.start(e, $(this).parents('.epg-mark'));
					self.timerResizeR.onDragMove = self.onTimerModified;
					
				})

				// drag marker 
				.on('mousedown touchstart', 'DIV.epg-mark-fn', function(e) {
					
					if ( !$(e.target).is('BUTTON') ) {
						
						e.stopPropagation();

						self.timerMove.start(e, $(this).parents('.epg-mark'));
						self.timerMove.onDragMove = self.onTimerModified;

					}
					
				})
				
				.appendTo($bar);			
				
			}
			
			loop++;
			if (!timer.reservationtype) {
				// one time
				break;
			} else if (timer.reservationtype == 3) {
				// weekly
				timer_start += 7*24*3600000;	// next week
				timer_end	+= 7*24*3600000;	// next week
				day			+= 7;				// next week
			} else {
				timer_start += 24*3600000;		// next day
				timer_end	+= 24*3600000;		// next day
				day++;							// next day
			}
			
		}

	};

	/**
	 * returns a jQuery object with channel logo and name
	 *
	 * @param {Object}	channel		channel object
	 * @param {Object}	status		status object
	 * @returns jqObject	(DIV.epg-head-channel)
	 */
	this.createChannelHead = function (channel, status) {

		var divClass = 'epg-head-channel epg-bg';
		if (status.current_logoid  == channel.logoid) {
			// this is the current channel
			divClass += ' current-channel';
		}

		// mark rec. channel
		$.each(status.recording, function(nr, recording) {
			if ( (recording.rectype) && (recording.logoid == channel.logoid) ) {
				divClass += ' rec-channel';
			}
		});

		var $template = $(
			'<div class="' + divClass + '" channelid="' + channel.logoid + '">' +
				'<div class="channellogo' +
				((channel.logo) ? ' logo-' + channel.logoid : '') +
				'"></div><br><span>' + channel.servicename + '</span>' +
			'</div>'
		).click( function() {
			// 'this' is the clicked element
			self.selectCurrentChannel( $(this) );
		});

		return $template;

	};

	/**
	 * returns a row container for channel
	 *
	 * @param {Object}	channel		channel object
	 * @param {Object}	status		status object
	 * @param {Array}	timers		Array of timer objects
	 * @returns jqObject	(DIV.epg-channel)
	 */
	this.createChannelRow = function (channel, status, timers) {

		var $line = $(
			'<div class="epg-channel epg-bg" channelid="' + channel.logoid + '">' +
				'<div class="epg-channel-marker epg-bg"></div>' +
			'</div>').prop('channel', channel);
		var $prgs = $('<div class="epg-channel-prgs"></div>').appendTo($line);

		// timers
		$.each(timers, function(ix, timer) {

			if (timer.logoid == channel.logoid) {

				self.createTimerMarker($line.find('DIV.epg-channel-marker'), timer);

			}

		});

		// recording slots
		$.each(status.recording, function(slot, recording) {

			if ( (recording.rectype) && (recording.logoid == channel.logoid) ) {

				var timer_start = new Date(recording.starttime).getTime();
				var timer_end	= new Date(recording.endtime).getTime();

				if (
						((timer_start	>= self.starttime) && (timer_start	<= self.endtime)) ||
						((timer_end		>= self.starttime) && (timer_end	<= self.endtime)) ||
						((timer_start	<= self.starttime) && (timer_end	>= self.endtime))
				) {

					var pos = Math.round( (timer_start - self.starttime) * self.pxPerMin / 60000 );
					var w	= Math.round( (timer_end - timer_start) * self.pxPerMin / 60000 );

					$line.find('DIV.epg-channel-marker').append(
							'<div class="epg-mark epg-bg epg-mark-recslot" style="left:' + pos + 'px;width:' + w + 'px;"></div>'
					);

				}

			}

		});

		// show event details
		$prgs.on('click', 'DIV.epg-channel-prg', function(e) {

			var $prg = $(this);

			// check, if any module handles the 'showEventInf' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showEventInfo) &&
					!$prg.hasClass('jk-mark')
			) {

				var event = $prg.prop('event');
				var channel = $prg.parents('DIV.epg-channel').prop('channel');

				tma.fn.showEventInfo( event, channel, $prg, function($oldprg) {
					$oldprg.removeClass('jk-mark');
				});

				$prg.addClass('jk-mark');

			}

		});

		return $line;

	};

	/**
	 * fills the timeline with the requested time range
	 *
	 * @param {Number}	starttime	in milliseconds
	 * @param {Number}	endtime		in milliseconds
	 */
	this.createTimeline = function (starttime, endtime) {

		$epgframe = $('DIV.epg-container');

		var tl_start = new Date( parseInt(starttime / 3600000) * 3600000 ).getTime();
		var tl_hours = parseInt( (endtime - tl_start + 3599999) / 3600000);

		var pos = Math.round( (tl_start - starttime) * this.pxPerMin / 60000 ) - 2;

		var template = '';
		for ( var i = 0; i < tl_hours; i++ ) {

			template +=
				'<div class="epg-hour-marker">' +
					'<div class="epg-date-marker">' + tma.date(this.i18.shortdate_fmt, tl_start) + '</div>' +
					'<div class="epg-time-marker">' +
						'<div class="epg-qtr-marker">' +
							tma.date('G:i', tl_start				) + '</div>' +
						'<div class="epg-qtr-marker">' +
							tma.date('G:i', tl_start + 15000 * 60	) + '</div>' +
						'<div class="epg-qtr-marker">' +
							tma.date('G:i', tl_start + 30000 * 60	) + '</div>' +
						'<div class="epg-qtr-marker">' +
							tma.date('G:i', tl_start + 45000 * 60	) + '</div>' +
					'</div>' +
				'</div>';

			tl_start += 3600000;
		}

		$epgframe.find('DIV.epg-timeline-frame DIV.epg-timeline').append(template).css({left: pos});

	};
	
	/**
	 * process the result of the EGP query
	 * 
	 * @param {Object}		epg				json-data
	 * @param {ChannelData} channelData
	 */
	this.processEpgResult = function(epg, channelData) {

		if (epg.event) {
			$.each(epg.event, function(ix, event) {

				if (!channelData.events[event.eventid]) {
					
					channelData.events[event.eventid] = event;
					
					var event_start_date= new Date(event.starttime_local);
					var event_end_date	= new Date(event.endtime_local);
					var event_start = event_start_date.getTime();
					var event_end	= event_end_date.getTime();

					var current = 	(event_start < self.starttime) &&
									(event_end	 > self.starttime) ?
											' epg-prg-current' : '';

					var pos = Math.round( (event_start - self.starttime) * self.pxPerMin / 60000 ) + 1;
					var w	= event.duration * self.pxPerMin; //+ 1;
					if (pos < 0) {
						w += pos;
						pos = 0;
					}

					if (w > 0) {
					
						var $prgs = $('DIV.epg-channel[channelid="' + channelData.channel.logoid + '"] DIV.epg-channel-prgs');
						$(
							'<div class="epg-channel-prg epg-bg' + current + '" ' +
								'style="left: ' + (pos - 1) + 'px;width: ' + w + 'px;min-width: ' + w + 'px;">' +
								'<span class="epg-prg-time">' + tma.date('H:i', event.starttime_local) + ' - ' +
									tma.date('H:i', event.endtime_local) + ' &nbsp; ' +
									event.duration + '&nbsp;min</span>' +
								'<span class="epg-prg-title">' + event.name + '</span>' +
								'<span class="epg-prg-descr">' + event.description + '</span>' +
							'</div>'
						).appendTo($prgs)
						.prop('event', event);
					}
					
				}

			});
		}

	};
	
	/**
	 * 
	 */
	this.isChannelVisible = function(channelid, scrollTop, clientHeight) {
		
		var $head = $('DIV.epg-head DIV.epg-head-channel[channelid="' + channelid + '"]');
		if ($head.length) {
			
			var top = $head.position().top;
			var height = $head.outerHeight();
			
			return (scrollTop < (top + height)) && (top < clientHeight + scrollTop);
			
		} else {
			return false;
		}
		
	};
	
	/**
	 * returns the channel data of a specific channel
	 * 
	 * @param {String}	channelid
	 * @returns {ChannelData}
	 */
	this.channelData = function(channelid) {
		
		return channelsData[channelid];
		
	};
	
	/**
	 * load and display EPG data of visible time slices in visible channels 
	 */
	this.updateEpg = function() {
		
		$container = $('DIV.epg-container');
		var scrollTop = $container.scrollTop();
		var scrollLeft = $container.scrollLeft();
		var clientHeight = $container.height() - $('DIV.epg-timeline-frame').outerHeight();
		var clientWidth = $container.width() - $('DIV.epg-head').outerWidth();
		
		// update current day
		var curr_time = this.starttime + Math.round(scrollLeft * 60000 / this.pxPerMin);
		$('DIV.epg-nav-days BUTTON.epg-nav-day').removeClass('epg-nav-current-day')
		.filter('BUTTON[date="' + tma.date('Ymd', curr_time) + '"]').addClass('epg-nav-current-day');

		// start timestamp in minutes
		var start = Math.round(this.starttime / 60000);
		// offset from slice start in pixel
		var offs = (start % this.timeSliceLength) * this.pxPerMin;
		
		// get visible time slices
		var fromSliceNr = parseInt( (scrollLeft + offs)					/ (this.pxPerMin * this.timeSliceLength) );
		var toSliceNr	= parseInt( (scrollLeft + clientWidth + offs) 	/ (this.pxPerMin * this.timeSliceLength) );

		// check sliceNr bounds
		fromSliceNr = Math.max(0, Math.min(fromSliceNr	, this.maxTimeSlices - 1));
		toSliceNr	= Math.max(0, Math.min(toSliceNr	, this.maxTimeSlices - 1));
		
		$.each(this.channels, function(ix, channel) {

			if (self.isChannelVisible(channel.logoid, scrollTop, clientHeight)) {
				
				for(var sliceNr = fromSliceNr; sliceNr <= toSliceNr; sliceNr++) {
					
					var slice = self.channelData(channel.logoid).slice(sliceNr);

					if (!slice.state) {
						
						slice.state = 1; // loading
						tma.apiGetEpg(channel.chtype, channel.chnum, slice.starttime, slice.endtime).done(function(epg) {
							slice.state = 2; // loaded
							slice.events = epg.count;
							
							self.processEpgResult(epg, self.channelData(channel.logoid));
						});
						
					}
					
				}
				
			}

		});		
		
	};
	
	/**
	 * renders the navigation bar
	 */
	this.createNavigationBar = function() {

		var template = '<div class="epg-nav">';

		template +=		'<button class="jk-but iconset ic-clock" title="' + this.i18.timer_mode_tooltip + '"></button>';
		template += 	'<button class="jk-but iconset ic-left"  title="' + '"></button>';
		template += 	'<button class="jk-but iconset ic-right" title="' + '"></button>';
		template += 	'<div class="epg-nav-days">';
		
		var days = parseInt( (this.maxTimeSlices + 1) * this.timeSliceLength / (24*60) );
		var date = this.starttime;
		for(var day = 0; day < days; day++, date += 24*3600000) {
			
			template += '<button class="jk-but epg-nav-day" day="' + day + '" date="' + tma.date('Ymd', date) + '">';
			template +=		tma.date(this.i18.shortdate_fmt, date);
			template += '</button>';		
		}

		template += 	'</div>';
		template += '</div>';
		
		return $(template).appendTo('DIV#content')
		.on('click', 'BUTTON.epg-nav-day', function() {
			
			var clickedDay = parseInt( $(this).attr('day') );
			var currentDay = parseInt( $('DIV.epg-nav-days BUTTON.epg-nav-current-day').attr('day') );
			
			var $container = $('DIV.epg-container');
			var diff = (clickedDay - currentDay) * self.pxPerMin * 24*60;
			$container.scrollLeft($container.scrollLeft() + diff);
			
		}).on('click', 'BUTTON.ic-left', function() {
			
			var $container = $('DIV.epg-container');
			var clientWidth = $container.width() - $('DIV.epg-head').outerWidth();
			
			$container.scrollLeft($container.scrollLeft() - clientWidth);
			
		}).on('click', 'BUTTON.ic-right', function() {
			
			var $container = $('DIV.epg-container');
			var clientWidth = $container.width() - $('DIV.epg-head').outerWidth();
			
			$container.scrollLeft($container.scrollLeft() + clientWidth);
			
		})		
		.on('click', 'BUTTON.ic-clock', function() {
			// toggle timermode
			$('DIV#content').toggleClass('mode-edit-timer');

		});

	};

	/**
	 * load and display an EPG grid
	 *
	 * currently the EPG data is loaded for all channels of the selected favourites group (=> channels page)
	 * the time range is currently fix 3 hours from now
	 */
	this.showEPG = function () {

		var favGroup = tma.setting('defaultfavourites', '');

		this.channels = tma.getChannels(favGroup);

		this.starttime = (tma.config.mock) ? new Date('2013/10/13 18:06:02') : new Date();
		this.starttime = Math.round(this.starttime / 60000) * 60000;
		var minutes = this.maxTimeSlices * this.timeSliceLength;
		this.endtime = this.starttime + minutes * 60000;

		this.createNavigationBar();
		
		var $template = $(
			'<div class="epg-container">' +
				'<div class="epg-channels" style="width: ' + (minutes * this.pxPerMin + 80) + 'px;">' +
					'<div class="epg-timeline-frame epg-bg-rev" style="width: ' + (minutes * this.pxPerMin + 80) + 'px;">' +
						'<div class="epg-timeline"></div>' +
					'</div>' +
					'<div class="epg-head"></div>' +
				'</div>' +
			'</div>').appendTo('DIV#content');

		// iOS hack
		$template.on('dragstart drop', /*'DIV',*/ function(e) {
			e.preventDefault();
		});

		// adjust vertical size (vScroll in EPG-frame)
		$(window).resize(function() {
			var offs = $template.offset();
			var maxH = /*$(document).scrollTop() +*/ $(window).height() - offs.top - 15;
			$template.css('max-height', maxH);
		}).resize();

		// keep channel on left border
		// and timeline on top (scroll in EPG-container)
		$template.scroll(function() {

			var scrollTop = $template.scrollTop();
			$template.find('DIV.epg-timeline').parent().css('top', scrollTop);

			var scrollLeft = $template.scrollLeft();
			$template.find('DIV.epg-head').css('left', scrollLeft);

			self.updateEpg();

		});

		// move content with mouse
		if ( !Modernizr.touch ) {
			var $ch_container = $template.find('DIV.epg-channels');
			var dragndrop = new jkqDragDrop();
			dragndrop.onDragStart = function(e, drag_source) {
				this.drag_x_rel = - $template.scrollLeft()	- e.pageX;
				this.drag_y_rel = - $template.scrollTop()	- e.pageY;
			};
			dragndrop.dragmove = function(x, y) {
				$template.scrollLeft(-x - this.drag_x_rel);
				$template.scrollTop( -y - this.drag_y_rel);
			};

			// start drag
			$ch_container.on('mousedown', function(emd) {
				// move
				$ch_container.on('mousemove', function(emm) {
					if (	Math.sqrt(
							Math.pow(emm.pageX - emd.pageX, 2) +
							Math.pow(emm.pageY - emd.pageY, 2)
						) > 5
					) {
						$ch_container.off('mousemove');
						dragndrop.start(emd, $ch_container);
					}
				});
			}).on('mouseup', function() {
				$ch_container.off('mousemove');
			});
		}

		// first get status to update current channel and rec channels
		tma.apiGetStatus().done(function(status) {

			// then get timers to set reservation-bars
			tma.apiGetTimers().done(function(timers) {
				
				// setup the timeline
				self.createTimeline( self.starttime, self.endtime );

				// create lines for each channel
				$.each(self.channels, function(ix, channel) {

					$template.find('DIV.epg-head'	 ).append( self.createChannelHead(channel, status) );
					$template.find('DIV.epg-channels').append( self.createChannelRow( channel, status, timers.timers) );
					
					channelsData[channel.logoid] = new ChannelData(channel, self.starttime, self.timeSliceLength, self.maxTimeSlices);

				});

				// adjust vertical size (vScroll in EPG-frame)
				$(window).resize(function() {

					var offs = $template.offset();
					var maxH = /*$(document).scrollTop() +*/ $(window).height() - offs.top - 15;
					$template.css('max-height', maxH);

					self.updateEpg();

				}).resize();

			});

		});

		// handle added timer
		$(tma).on('tmaTimerAdded.epgpage', function(e, timer) {

			// add reservation 
			var $bar = $('DIV.epg-channel[channelid="' + timer.logoid + '"] DIV.epg-channel-marker');
			self.createTimerMarker($bar, timer);

		});

		// handle modified timer
		$(tma).on('tmaTimerModified.epgpage', function(e, timer) {

			// remove prev reservetion
			$('DIV.epg-mark[timerindex="' + timer.index + '"]').remove();
			// add modified reservation 
			var $bar = $('DIV.epg-channel[channelid="' + timer.logoid + '"] DIV.epg-channel-marker');
			self.createTimerMarker($bar, timer);

		});

	};

	/**
	 * selects a channel and marks the element with css class 'current-channel'
	 *
	 * @param {jQueryObject} $channel	channel element with attribute channelid
	 */
	this.selectCurrentChannel = function ($channel) {

		var channel;
		if ( channel = tma.channel( $channel.attr('channelid') ) ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).done(function() {
				// set new current channel mark
				$('.current-channel').removeClass('current-channel');
				$channel.addClass('current-channel');
			}).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.dialog_title_error,
						text:	self.i18.error_channel_set
					}).confirm();
				}

			});
		}

	};

	this.insertConfig = function () {

	};
	
	// init reservation resize
	
	// drag left end, to expand reservation
	this.timerResizeL = new jkqResizeLeft();
	
	// drag right end, to expand reservation
	this.timerResizeR = new jkqResizeRight();

	// move reservation
	this.timerMove = new jkqDragDrop();

}

function TimeSlice() {
	
	/**
	 * query state:
	 * 	0: empty
	 * 	1: loading
	 *  2: ready 
	 */
	this.state = 0; // empty
	
	/**
	 * count of received events (state=2)
	 */
	this.events = 0;
	
	/**
	 * slice start in PVR date format 
	 */
	this.starttime;
	
	/**
	 * slice end in PVR date format 
	 */
	this.endtime;
	
}

/**
 * @param {Object}	channel
 * @param {Date}	starttime	timestamp of the grids starttime
 * @param {int}		sliceLength	minutes per slice
 * @param {int}		maxSlices	max. number of timeslices
 */
function ChannelData(channel, starttime, sliceLength, maxSlices) {
	
	this.channel = channel;
	
	var slices = {};
	
	this.events = {};
	
	/**
	 * returns a time slice
	 * 
	 * @param {Number}	channelid
	 * @returns {TimeSlice}
	 */
	this.slice = function(sliceNr) {
		
		return slices[sliceNr];
		
	};
	
	// init slices
	var start = starttime;
	sliceLength *= 60000;
	var time = start - (start % sliceLength);
	for (var i = 0; i < maxSlices; i++) {
		
		var slice = new TimeSlice();
		slice.starttime = tma.toPvrDate( Math.max(time, start) );
		time += sliceLength;
		slice.endtime = tma.toPvrDate(time);
		slices[i] = slice;
		
	}
	
}
