/**
 * EPG event info modul
 */
tma.registerModule( eventinfo );
eventinfo.prototype = new tmaModule();
function eventinfo() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'eventinfo';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'showEventInfo' );

		return this.parent().setup();

	};

	/**
	 * Show inf data of a recording
	 *
	 * @param {tmaFeature}	f
	 * @param {Object}		event		tma EPG event object
	 * @param {Object}		channel		tma channel object
	 * @param {any}			data		custom data
	 * @param {Function}	onClose		callback function(data), that is called when the info gets closed
	 */
	this.showEventInfo = function (f, event, channel, data, onClose) {

		/**
		 * this method will handle the feature,
		 * so stop the feature chain
		 */
		f.stop();

		this.init().done(function() {

			var eventWindow = new jkqWindow();
			eventWindow.width = 500;

			if (onClose) {
				// send back current file object
				eventWindow.onClose = function() {
					onClose(data);
				};
			} else {
				eventWindow.onClose = null;
			}

			eventWindow.show();
			eventWindow.setTitle( channel.servicename );
			eventWindow.closeOnOutsideClick();

			var $content = eventWindow.container.find('DIV.jkqWindowContent');

			var event_start 	= new Date(event.starttime_local);
			var event_end		= new Date(event.endtime_local);
			var event_duration	= parseInt( (event_end.getTime() - event_start.getTime()) / 60000);

			$template = $(
					'<div class="eventinfo-frame">' +
						'<div class="eventinfo-top-bar">' +
							'<div class="sign-rec" />' +
							'<button class="jk-but iconset ic-clock" title="' + self.i18.edit_timer_tooltip + '" />' +
							'<button class="jk-but iconset ic-rec" title="' + self.i18.create_timer_tooltip + '" />' +
							'<button class="jk-but iconset ic-gear" />' +
						'</div>' +
						'<div class="config eventinfo-config">' +
							'<div class="setting setting-less-more pretime">' +
								'<span class="label">' + self.i18.config.pretime + '</span>' +
								'<input type="text" name="pretime" value="0" disabled> <span>min.</span> ' +
								'<button class="lessmore less">&#x25C0;</button>' +
								'<button class="lessmore more">&#x25B6;</button>' +
							'</div>' +
							'<div class="setting setting-less-more posttime">' +
								'<span class="label">' + self.i18.config.posttime + '</span>' +
								'<input type="text" name="posttime" value="0" disabled> <span>min.</span> ' +
								'<button class="lessmore less">&#x25C0;</button>' +
								'<button class="lessmore more">&#x25B6;</button>' +
							'</div>' +
						'</div>' +
						'<div class="eventinfo">' +
							getEventDateString(event_start, event_end, event_duration) +
							'<h1>' + event.name + '</h1>' +
							((channel) && (channel.logo) ? '<div class="channellogo logo-' + channel.logo.logoid + '"/>' : '') +
							getGenreAndParentalString(event) +
							'<h2>' + event.description + '</h2>' +
							'<p>' + event.text + '</p>' +
						'</div>' +
					'</div>'
			).appendTo($content);

			// is "edittimer" available
			if (tma.fn.editTimer) {

				// assign timer to event, if available
				event.timer = self.findTimer(event);

				// edit the timer
				$template.find('DIV.eventinfo-top-bar BUTTON.ic-clock').click( function() {

					tma.fn.editTimer(event.timer);

				});

				// create a timer
				$template.find('DIV.eventinfo-top-bar BUTTON.ic-rec').click( function() {

					var pretime	= parseInt(self.setting('pretime'	, '0'));
					var posttime= parseInt(self.setting('posttime'	, '0'));

					// create a new timer
					tma.fn.editTimer(
							new TmaTimer({
								index:		-1,
								isrec:		1,
								starttime:	tma.toPvrDate(event_start.getTime() - (pretime * 60000) ),
								duration:	event_duration + pretime + posttime,
								filename:	event.name + tma.config.recextension,
								logoid:		channel.logoid,
								eventid1:	event.eventid,
								eventid2:	event.eventid,
								serviceid:	event.serviceid,
							}),
							{
								onsaved: function(timer) {
									event.timer = timer;
									$template.addClass('has-timer');
								}
							}
					);

				});

				// show a record sign, if there is a timer with the eventid of this event
				if (event.timer) {

					$template.addClass('has-timer');

				}
			}

			/* config */
			$template.on('click', 'BUTTON.ic-gear', function() {
				$template.find('DIV.eventinfo-config').slideToggle();
			});

			// config pretime setting
			var $pretime = $template.find('DIV.eventinfo-config DIV.pretime INPUT');
			$pretime.val( self.setting('pretime', '0') );

			$template.find('DIV.eventinfo-config DIV.pretime BUTTON.less').click(function() {

				var val = parseInt( $pretime.val() );
				if (val > 0) {
					$pretime.val( (val - 1) );
					self.setSetting('pretime', val - 1);
				}

			});
			$template.find('DIV.eventinfo-config DIV.pretime BUTTON.more').click(function() {

				var val = parseInt( $pretime.val() );

				$pretime.val( (val + 1) );
				self.setSetting('pretime', val + 1);

			});

			// config posttime setting
			var $posttime = $template.find('DIV.eventinfo-config DIV.posttime INPUT');
			$posttime.val( self.setting('posttime', '0') );

			$template.find('DIV.eventinfo-config DIV.posttime BUTTON.less').click(function() {

				var val = parseInt( $posttime.val() );
				if (val > 0) {
					$posttime.val( (val - 1) );
					self.setSetting('posttime', val - 1);
				}

			});
			$template.find('DIV.eventinfo-config DIV.posttime BUTTON.more').click(function() {

				var val = parseInt( $posttime.val() );

				$posttime.val( (val + 1) );
				self.setSetting('posttime', val + 1);

			});


			eventWindow.adjustPos({
				hAlign:	'right',
				effect:	'slide-rtl'
			});

		});

	};

	/**
	 * searches a timer with a specific eventid
	 *
	 * @param {Object}	event	tma-Event object
	 * @returns {Object} tma-Timer object, or false if not found
	 */
	this.findTimer = function(event) {

		var found = false;

		$.each(tma.timers.timers, function(nr, timer) {

			if (timer.serviceid == event.serviceid) {

				var timer_start = new Date(timer.starttime).getTime();
				var timer_end	= new Date(timer.endtime).getTime();
				var event_start = new Date(event.starttime_local);
				var event_end	= new Date(event.endtime_local);

				if (timer.eventid2 == event.eventid) {
					// found by corresponding eventid
					found = timer;
					return false; // break
				} else if ((timer_start <= event_start) && (timer_end >= event_end)) {
					// found: event is fully covered
					found = timer;
					return false; // break
				} else if (
						((timer_start <= event_start) && (timer_end > event_start))  ||
						((timer_start < event_end) && (timer_end >= event_end))
				) {
					// found: event is partially covered
					found = timer;
					return false; // break
				}
			}

		});

		return found;

	};

}
