/**
 * RecInf modul
 */
tma.registerModule( recfileinfo );
function recfileinfo() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recfileinfo';

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'showRecInf' );

		return this.parent().setup();

	};

	/**
	 * returns the html string for a recording bar
	 *
	 * @param {Boolean}	partial
	 * @param {Number}	startBlock
	 * @param {Number}	startPercent
	 * @param {Number}	durationPercent
	 * @returns {String}
	 */
	this.recBar = function (partial, startBlock, startPercent, durationPercent) {

		var s =	'<div class="recbar">' +
		'<div class="eventbar" partial="' + partial + '" ' +
		'startblock="' + startBlock + '" ' +
		'style="left:' + startPercent + '%;width:' + durationPercent + '%;" ' +
		'title="' + this.i18.event_bar_title + '">' +
		'&nbsp;&#x25B6;</div>' +
		'</div>';

		return s;

	};

	/**
	 * display standard recording file info
	 *
	 * @param {tmaFeature}	f
	 * @param {Object}		file		tma recfile object
	 * @param {Function}	onClose	callback function, that is called when the info gets closed
	 */
	this.showRecInf = function(f, file, onClose) {

		if (tma.debug) tma.log('recfileinfo.showRecInf');
		if (tma.debug) tma.log('recfileinfo.showRecInf: typeof c=' + typeof(c));
		if (tma.debug) tma.log('recfileinfo.showRecInf: typeof f=' + typeof(f));
		if (tma.debug) tma.log('recfileinfo.showRecInf: typeof onClose=' + typeof(onClose));
		if (tma.debug) tma.log('recfileinfo.showRecInf: file=' + file.name);

		/**
		 * this method will handle the feature,
		 * so stop the feature chain
		 */
		f.stop();

		this.init().done(function() {

			if (tma.debug) tma.log('recfileinfo.showRecInf: init.done');

			var channel = tma.channel(file.infdata.header.logoid);

			var infoWindow = new jkqWindow();
			infoWindow.width = 500;

			if (onClose) {
				// send back current file object
				infoWindow.onClose = function() {
					onClose(file);
				};
			} else {
				infoWindow.onClose = null;
			}

			infoWindow.show();
			infoWindow.setTitle( (channel) ? channel.servicename : file.infdata.service.servicename );
			infoWindow.closeOnOutsideClick();

			var $content = infoWindow.container.find('DIV.jkqWindowContent');

			var recStart = new Date(file.infdata.header.starttime);
			var recDuration = file.infdata.header.durationmin * 60000 + file.infdata.header.durationsec * 1000;
			var recEnd = new Date(recStart.getTime() + recDuration);
			var recBlocks = parseInt(file.size / tma.config.blockfactor);

			var eventStart 		= new Date(file.infdata.eventinfo.starttime_utc + ' UTC');
			var eventEnd		= new Date(file.infdata.eventinfo.endtime_utc + ' UTC');
			var eventDuration	= parseInt( (eventEnd.getTime() - eventStart.getTime()) / 60000);

			var calcStart		= Math.max(recStart.getTime(), eventStart.getTime());
			var calcEnd			= Math.min(recEnd.getTime(), eventEnd.getTime());
			var calcDuration	= parseInt((calcEnd - calcStart) / 60000);

			var startPercent	= parseInt( (calcStart - recStart) * 1000 / recDuration ) / 10;
			var durationPercent	= parseInt( (calcEnd - calcStart) * 1000 / recDuration ) / 10;

			var partial = (eventDuration != calcDuration);

			var startBlock = parseInt(recBlocks * startPercent / 100);

			$(
					'<div class="inf">' +
					'<div class="fileinfo">' +
					getEventDateString(eventStart, eventEnd, eventDuration, calcDuration) +
					self.recBar(partial, startBlock, startPercent, durationPercent) +
					'<h1>' + file.infdata.eventinfo.name + '</h1>' +
					((channel) && (channel.logo) ? '<div class="channellogo logo-' + channel.logo.logoid + '"/>' : '') +
					getGenreAndParentalString(file.infdata.eventinfo) +
					'<h2>' + file.infdata.eventinfo.description + '</h2>' +
					'<p>' + file.infdata.exteventinfo.text + '</p>' +
					'</div>' +
					'</div>'
			).appendTo($content)
			// play event
			.on('click', 'DIV.eventbar', function() {

				var startposition = $(this).attr('startblock');

				if (tma.fn.playFile) {
					tma.fn.playFile( tma.files.directory, file, startposition );
				}

			});

			if (tma.debug) tma.log('recfileinfo.showRecInf: window.adjustPos');

			if (tma.debug) {
				infoWindow.adjustPos();
			} else {
				infoWindow.adjustPos({
					hAlign:	'right',
					effect:	'slide-rtl'
				});
			}

		});

	};

}
