/**
 * recordings module
 */
tma.registerModule( recordingspage );
recordingspage.prototype = new tmaModule();
function recordingspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recordingspage';
	
	this.pageGroup = 'files';

	// available table columns
	var tableColumns = {
			filename:	true,
	        filefn:		true,
	        filesize:	true,
	        filedate:	true,
	        duration:	true,
	        channel:	true
	        };

	var self = this;

	/**
	 * js timer, checks status periodically (set by this.playFile)
	 */
	this.statusInterval = null;

	/**
	 * setup module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'playFile' );

		return this.parent().setup();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.addPage( this, this.i18.menu_caption );
			
		}

	};

	/**
	 * display the content
	 */
	this.show = function() {

		this.init().done(function() {

			if (tma.files == null) {
				// load the files
				var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
				self.loadFiles(dir);
			} else {
				// show existing files
				self.showFiles();
			}
			
			/** @returns MenuManager */
			var menu;
			if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

				menu.insertItemAfter(
						self.name,
						self,
						'recordings-config',
						'<i class="fa fa-gear" />',
						self.toggleConfig
				).addClass('config');
				
			}

			self.insertConfig();

		});

	};

	/**
	 * reload data and display the content
	 */
	this.refresh = function() {

		// refresh the files
		var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
		self.loadFiles(dir);

	};

	/**
	 * this page gets removed, so display #tma-status again
	 */
	this.cleanup = function() {

		// stop status checking
		if (self.statusInterval) {
			window.clearTimeout(self.statusInterval);
			self.statusInterval = null;
		}

		// hide and clear config frame, if exists
		$('DIV#config').hide().empty().removeClass('recordings-config');
		
		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.removeItem('recordings-config');
			
		}

	};

	/**
	 * (de)activate config
	 */
	this.toggleConfig = function() {

		$('DIV#config').slideToggle();

	};

	/**
	 * get a new directory content
	 */
	this.loadFiles = function (directory) {

		// delete all child nodes of content and display a pending image
		$('#content').append('<img src="css/img/wait.gif">');
		$("#timestamp").text(this.i18.files_loading);

		tma.apiGetFiles(directory).then(function() {

			// remove pending image
			$('#content').empty();

		}).done(function() {

			self.showFiles();

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").append( self.i18.load_error.format(errorType, error) );

		});

	};

	/**
	 * display path breadcrumbs
	 */
	this.renderBreadcrumbs = function() {

		var path = tma.files.directory.replace(/^\//, '').split('/');
		var $dirDiv = $(
				'<div class="directory">' +
					'<select name="mount">' +
						'<option value="/mnt/hd/DataFiles">DataFiles</option>' +
						'<option value="/mnt/hd/MediaFiles">MediaFiles</option>' +
						//'<option value="/mnt/hd/MP3Files">MP3Files</option>' +
						//'<option value="/mnt/hd/PhotoFiles">PhotoFiles</option>' +
					'</select><br>' +
				'</div>'
		)
		.appendTo('DIV#content')
		.on('click', 'A', function(e) {
			e.preventDefault();
			self.loadFiles( $(this).attr('href') );
		}).on('change', 'SELECT', function(e) {
			self.loadFiles( $(this).val() );
		});

		var mount = '';
		var mount_depth = 1;
		$.each(path, function(nr, value) {

			mount += '/' + value;
			$('<span>/</span>').appendTo($dirDiv);

			if ( (nr == 1) && (value == 'hd')) {
				mount_depth = 2;
			}

			if ( nr >= mount_depth ) {

				$('<a href="' + mount + '">' + value + '</a>').appendTo($dirDiv);

				if ( nr == mount_depth ) {
					$dirDiv.find('SELECT').val(mount);
				}

			} else {

				$('<span>' + value + '</span>').appendTo($dirDiv);

			}

		});

	};

	/**
	 * display the file list of the current files object
	 */
	this.showFiles = function () {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.files.timestamp).toLocaleString() );
		$("#counter"	).text( tma.i18.record_counter.format( tma.files.count ) );

		// show disk space
		$('<div/>', {
			'class':	'diskspace'
		}).appendTo('DIV#content');

		this.setFilesStatus();

		if (!tma.files) return;

		// sort files
		var sortKey = this.localSetting('sort_key', 'name');
		var sortDir = this.localSetting('sort_dir', 'asc');
		tma.sortFiles(sortKey, sortDir);

		// display path breadcrumbs
		this.renderBreadcrumbs();

		// create Table
		var $table = $('<table/>', {
			'class': "files jk-list jk-round"
		});

		var head = '<thead><tr>';
		var row  = '<tr>';

		var userCols = this.localSetting( 'columns', tableColumns );
		$.each(userCols, function(clmClass) {
			head += '<th class="' + clmClass + '" can-sort-dir="asc"><a/></th>';
			row  += '<td class="' + clmClass + '"><span/></td>';
		});

		head +=	'</tr></thead>';
		row  +=	'</tr>';

		// create table head with sort options
		var $head = $(head).appendTo($table);
		$head.find('TH.filename A').text(this.i18.columns.filename);
		$head.find('TH.filesize A').text(this.i18.columns.filesize);
		$head.find('TH.filedate A').text(this.i18.columns.filedate);
		$head.find('TH.duration A').text(this.i18.columns.duration);
		$head.find('TH.channel  A').text(this.i18.columns.channel);
		$head.find('TH.filefn   A').remove();

		$head.find('TH.filename').attr('sort-key', 'name');
		$head.find('TH.filesize').attr('sort-key', 'size');
		$head.find('TH.filedate').attr('sort-key', 'sortdate');
		$head.find('TH.duration').attr('sort-key', 'infdata.header.durationmin');
		$head.find('TH.channel' ).attr('sort-key', 'infdata.service.servicename');
		$head.find('TH.filefn'	).removeAttr('can-sort-dir');

		// set the current sort state
		$head.find('TH[sort-key="' + sortKey + '"]').attr({
			'sort-dir':		sortDir,
			'can-sort-dir': (sortDir == 'asc') ? 'desc' : 'asc'
		});

		// set sort event handler
		$head.on('click', 'A', function(e) {
			e.preventDefault();
			var sDir = $(this).parent().attr('can-sort-dir');
			var sKey = $(this).parent().attr('sort-key');
			self.setLocalSetting('sort_key', sKey);
			self.setLocalSetting('sort_dir', sDir);
			self.showFiles();
		});

		var $tbody = $('<tbody/>').appendTo($table);

		// create file list
		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var channel;

		for(var i = 0; i < tma.files.files.length; i++) {

			var f = tma.files.files[i];

			$row = $(row).appendTo($tbody);

			$row.attr('attr', f.attr);
			$row.attr('inode', f.inode);
			$row.prop('file', f);

			var name;
			if (tma.config.utftoppy || !f.name_hex) {
				name = f.name;
			} else {
				name = tma.decodeHexname(f.name_hex);
			}
			$row.find('TD.filename SPAN').text( tma.removeExtension(name) );

			if (f.attr == 208) {
				// size
				if (		f.size /       1024 /*1KB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( ( f.size ).toLocaleString() + ' B');
				} else if (	f.size /    1048576 /*1MB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 /       1024 /*1KB*/) / 10 ).toLocaleString() + ' KB');
				} else if (	f.size / 1073741824 /*1GB*/ < 0.1 ) {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 /    1048576 /*1MB*/) / 10 ).toLocaleString() + ' MB');
				} else {
					$row.find('TD.filesize SPAN').text( (parseInt(f.size * 10 / 1073741824 /*1GB*/) / 10 ).toLocaleString() + ' GB');
				}

				// date
				if ( (f.infpresent) && (f.infdata.header.starttime)) {
					$row.find('TD.filedate SPAN').text( tma.date( this.i18.recdate_fmt, f.infdata.header.starttime ) );
				} else {
					$row.find('TD.filedate SPAN').text( tma.date( this.i18.recdate_fmt, f.timestamp ) );
				}

				if (f.infpresent) {

					//$row.addClass('playable');

					// duration
					$row.find('TD.duration SPAN').text( tma.formatDuration(f.infdata.header.durationmin) );

					// channel
					if ( channel = tma.channel(f.infdata.header.logoid) ) {

						$row.find('TD.channel').html(
								'<div class="channellogo-small logo-small-' + channel.logoid + '" />' +
								'<span class="channelname">' +
								((channel) ? channel.servicename : f.infdata.service.servicename) +
								'</span>'
						);
					}

					// info button
					$('<button class="jk-but info">i</button>').appendTo($row.find('TD.filefn > SPAN'));

				//} else if ( f.name.match(/\.rec$|\.mpg$|\.mkv$|\.ts$|\.vob$|\.m2ts$|\.divx$/) ) {

					//$row.addClass('playable');

				}

				// more functions opener
				$('<button class="jk-but iconset ic-gear morefn" title="more functions"></button>').appendTo($row.find('TD.filefn > SPAN'));

			}

		}

		// hide logo or channel name
		if ( (channelDisplay & 1) == 0 ) $table.find('TD.channel > DIV').hide();
		if ( (channelDisplay & 2) == 0 ) $table.find('TD.channel > SPAN').hide();

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (!display) $table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
		});

		// add event listener for folders to change directories
		$table.find('TR[attr="224"]').on('click', function() {

			// 'this' is the clicked element
			var f = $(this).prop('file');
			var directory = tma.files.directory + '/' + f.name;
			self.loadFiles( directory );

		});

		// add event listener to show infos
		$table.on('click', 'BUTTON.info', function(e) {

			var $row = $(this).parents('TR');

			if (tma.debug) tma.log('recordingspage.info.click');

			// check, if any module handles the 'showRecInf' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showRecInf) &&
					!$row.hasClass('jk-mark')
			) {

				if (tma.debug) tma.log('recordingspage.info.click: showRecInf available');

				var f = $row.prop('file');

				if (tma.debug) tma.log('recordingspage.info.click: f=' + f.name);

				tma.fn.showRecInf( f, function(oldf) {
					if (tma.debug) tma.log('recordingspage.info.click: callback f=' + oldf.name);
					$('TABLE.files TR[inode="' + oldf.inode + '"]').removeClass('jk-mark');
				});

				$row.addClass('jk-mark');

			}

		});

		// add event listener to show "more functions"
		$table.on('click', 'BUTTON.morefn', function() {

			// remove previous moreFn�s
			$('DIV.morefn').fadeOut(function() {
				$(this).remove();
			});

			$(this).parents('TR').addClass('jk-mark');

			$('<div class="morefn" />')
			// close
			.append('<button class="jk-but morefn_close"><span>&#x25B2;</span></button>')
			// play
			.append('<button class="jk-but iconset play"><span>' 		+ self.i18.menu.play +		'</span></button>')
			// stop
			.append('<button class="jk-but iconset stop"><span>'		+ self.i18.menu.stop +		'</span></button>')
			// stop recording
			.append('<button class="jk-but iconset stoprec"><span>'		+ self.i18.menu.stoprec +	'</span></button>')
			// delete
			.append('<button class="jk-but iconset ic-delete"><span>'	+ self.i18.menu.del	+		'</span></button>')
			// rename
			.append('<button class="jk-but iconset ic-rename"><span>'	+ self.i18.menu.rename +	'</span></button>')
			// move
			.append('<button class="jk-but iconset ic-move"><span>'		+ self.i18.menu.move +		'</span></button>')

			.appendTo( $(this).parent() )
			.fadeIn();

			self.setFilesStatus(status);

		});

		// add event listener to play files
		$table.on('click', 'BUTTON.play', function() {

			var $tr = $(this).parents('TR');
			var f = $tr.prop('file');

			var startposition = 0;
			if (f.infpresent) {
				startposition = f.infdata.bookmarks.startposition || 0;
			}

			self.playFile(null, tma.files.directory, f, startposition);

		});

		// add event listener to stop playback
		$table.on('click', 'BUTTON.stop', function() {

			var $tr = $(this).parents('TR');
			tma.stopPlayback().done(function() {

				$tr.removeClass('play');

				if (self.statusInterval) {
					window.clearTimeout(self.statusInterval);
					self.statusInterval = null;
				}

			});

		});

		// add event listener to stop recording
		$table.on('click', 'BUTTON.stoprec', function() {

			var $tr = $(this).parents('TR');
			var slot = parseInt( $tr.attr('slot') );
			tma.stopRecording(slot).done(function(data) {

				if (data.return_code == 1) {
					$tr.removeClass('rec');
				}

			});

		});

		// add event listener to delete files
		$table.on('click', 'BUTTON.ic-delete', function() {

			// 'this' is the clicked element
			var $tr = $(this).parents('TR');
			var filename = $tr.find('TD.filename').text();
			var f = $tr.prop('file');

			new jkqDialog({
				title: self.i18.delete_confirm_caption,
				text: self.i18.delete_confirm.format(filename),
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'ok') {

					tma.apiDeleteFile(tma.files.directory, f.name, f.name_hex).done(function(delObject) {

						if (
								( delObject.mpg_status == 0) &&
								((delObject.inf_status == 0) || (!f.infpresent)) &&
								((delObject.nav_status == 0) || (!f.infpresent) || (f.infdata.header.svctype))
						) {

							$tr.remove();

							self.setFilesStatus();

						} else {

							new jkqAlert({
								title:	tma.i18.dialog_title_error,
								text:	self.i18.delete_failed.format(
										delObject.mpg_status,
										delObject.inf_status,
										delObject.nav_status
								)
							}).confirm();

						}

					});

				}

			});

		});

		// add event listener to rename files
		$table.on('click', 'BUTTON.ic-rename', function() {

			var $row = $(this).parents('TR');
			var f = $row.prop('file');
			var name;
			if (tma.config.utftoppy || !f.name_hex) {
				name = tma.removeExtension(f.name);
			} else {
				name = tma.removeExtension(tma.decodeHexname(f.name_hex));
			}

			new jkqInputDialog({
				title: self.i18.rename_dialog_caption,
				text: self.i18.rename_dialog_text.format(name),
				inputValue: name.replace(/"/, '&quot;')
			}).confirm(function() {

				//alert(this.value());
				tma.apiRenameFile(
						tma.files.directory, f.name, f.name_hex,
						tma.files.directory, this.value() + tma.config.recextension
				).done(function(renameObject) {

					if (
							( renameObject.mpg_status == 0) &&
							((renameObject.inf_status == 0) || (!f.infpresent)) &&
							((renameObject.nav_status == 0) || (!f.infpresent) || (f.infdata.header.svctype))
					) {

						var new_name;
						if (tma.config.utftoppy || !f.name_hex) {
							new_name = f.name;
						} else {
							new_name = tma.decodeHexname(f.name_hex);
						}
						$row.find('TD.filename SPAN').text( tma.removeExtension(new_name) );

					} else {

						new jkqAlert({
							title: tma.i18.dialog_title_error,
							text: self.i18.rename_failed.format(
									renameObject.mpg_status,
									renameObject.inf_status,
									renameObject.nav_status
							)
						}).confirm();

					}

				});

			});

		});

		// add event listener to move files
		$table.on('click', 'BUTTON.ic-move', function() {

			// 'this' is the clicked element
			var $tr = $(this).parents('TR');
			var root = $('DIV.directory SELECT').val();

			// open a directory selector...
			if (tma.featureExist('selectDirectory')) {
				tma.fn.selectDirectory( root, function(selectedPath) {
					self.moveFile($tr, selectedPath);
				});
			}

		});

		// close "more functions"
		$table.on('click', 'DIV.morefn BUTTON', function() {
			$(this).parent().fadeOut(function() {
				$(this).remove();
			});
			$(this).parents('TR').removeClass('jk-mark');
		});

		$('DIV#content').append($table);

	};

	/**
	 * loads the status via API and updates diskspace and file's play/rec state
	 */
	this.setFilesStatus = function () {

		tma.apiGetStatus().done(function(status) {

			var $diskDiv = $('DIV.diskspace').empty();

			var freeGB = status.freedisk >> 10;
			var totalGB = status.totaldisk >> 10;
			var usedGB = totalGB - freeGB;

			$('<span>' + self.i18.disk_space + ' </span>').appendTo($diskDiv);

			$('<div/>', {
				'class':	'diskspace-used',
				html:		usedGB + ' GB'
			}).appendTo($diskDiv).css('width', (parseInt(usedGB * 200 / totalGB) / 10) + 'em');

			$('<div/>', {
				'class':	'diskspace-free',
				html:		freeGB + ' GB'
			}).appendTo($diskDiv).css('width', (parseInt(freeGB * 200 / totalGB) / 10) + 'em');

			$diskDiv.append('<span class="diskspace-total">' + totalGB + ' GB ' + self.i18.disk_total + '</span>');

			$('TABLE.files TBODY TR').removeClass('rec play');

			if (self.statusInterval) {
				window.clearTimeout(self.statusInterval);
				self.statusInterval = null;
			}

			if (status.absolutefilename) {
				// file is playing
				var file = tma.splitPath(status.absolutefilename);
				if (file) {
					if (tma.files.directory == file.directory) {
						$('TABLE.files TR').filter(function() {
							var f = $(this).prop('file');
							return (f) && (
									(f.name_hex + '2e696e66' /* .inf */ == status.filename_hex) ||
									(
											!status.filename.match(/\.inf$/) &&
											(f.name_hex == status.filename_hex)
									)
							);
						}).addClass('play');
					}
				}

				// periodically check the status
				self.statusInterval = window.setTimeout(
					function() {
						self.setFilesStatus();
					},
					5000
				);

			}

			if (tma.files.directory == '/mnt/hd/DataFiles') {

				$.each(status.recording, function(nr, recording) {

					if (recording.rectype) {

						$('TABLE.files TR').filter(function() {
							var f = $(this).prop('file');
							return (f) && (f.name_hex == recording.filename_hex);
						}).addClass('rec').attr('slot', recording.slot);

					}

				});

			}

			// remove external mount points
			$('DIV.directory SELECT OPTION[external]').remove();

			if (status.drive_count > 0) {
				var $dirSelect = $('DIV.directory SELECT');
				$.each(status.drives, function(nr, drive) {

					$('<option value="' + drive.mountpath + '" external="true">' + drive.mountpath + '</option>').appendTo($dirSelect);

					if ( tma.files.directory.indexOf(drive.mountpath) == 0 ) {
						// this is the current drive
						$dirSelect.val(drive.mountpath);

						// hide used and free space
						$('DIV.diskspace DIV').hide();

						// set total space
						$('DIV.diskspace SPAN.diskspace-total').html((drive.size_mb >> 10) + ' GB ' + self.i18.disk_total);

					}

				});
			}

		});

	};

	this.moveFile = function ($row, path) {

		var f = $row.prop('file');

		if (tma.files.directory != path) {

			tma.apiRenameFile(
					tma.files.directory, f.name, f.name_hex,
					path, f.name
			).done(function(moveObject) {

				if (
						moveObject.mpg_status == 0 &&
						moveObject.inf_status == 0 &&
						moveObject.nav_status == 0
				) {

					$row.remove();

				} else {

					new jkqAlert({
						title: tma.i18.dialog_title_error,
						text: self.i18.move_failed.format(moveObject.mpg_status, moveObject.inf_status, moveObject.nav_status)
					}).confirm();

				}

			});
		}

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({filefn:	'Buttons'}, this.i18.columns),
				'#content TABLE.files',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				'#content TABLE.files',
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
				}
		);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.addClass('recordings-config');


	};

	/**
	 * Feature handler: play a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {Object}		file				tma file object
	 * @param {Number}		startposition
	 */
	this.playFile = function (f, directory, file, startposition) {

		tma.apiPlayFile(directory, file.name, file.name_hex, startposition).done(function(playObject) {

			if (playObject.status == 0) {

				$('TABLE.files TR').removeClass('play')
					.filter('TR[inode="' + file.inode + '"]').addClass('play');

				// periodically check the status
				self.statusInterval = window.setTimeout(
					function() {
						self.setFilesStatus();
					},
					2000
				);

			} else {

				new jkqAlert({
					title: tma.i18.dialog_title_error,
					text: 'status ' + playObject.status + ': play file "' + playObject.requested_file + '" failed!'
				}).confirm();

			}

		});

	};

};
