/**
 * edit timer popup module
 */
tma.registerModule( timeredit );
function timeredit() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timeredit';

	var self = this;

	var editTimerWindow = new jkqWindow();

	this.options = {};

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'editTimer' );

		return this.parent().setup();

	};

	/**
	 * display the reservation form
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaTimer}	editTimer	tma timer object
	 * @param {Object}		options		edit timer options, e.g.:
	 * 		onsaved		callback function(updatedTimer), that is called, when the reservation is saved
	 * @returns {jQueryDeferred}
	 */
	this.editTimer = function(f, editTimer, options) {

		/**
		 * this method will handle the feature,
		 * so stop the feature chain
		 */
		f.stop();

		this.options = options || {};

		var promise = this.init().done(function() {

			// JKIT - prepare some strings for the template
			var formCaption = (editTimer.index < 0) ? self.i18.caption_new : self.i18.caption_edit;
			var updCaption  = (editTimer.index < 0) ? self.i18.button_create : self.i18.button_update;

			//Load a list of available channels.
			//TODO - Allow for PVRs with analogue input
			var channelOptions = '';
			var channels = tma.getChannels('');
			$.each(channels, function(nr, channel) {
				channelOptions += '<option value="' + channel.logoid + '">' + channel.servicename + '</option>';
			});

			// init the window
			editTimerWindow.showModal();
			editTimerWindow.setTitle( formCaption );

			var $content = editTimerWindow.container.find('DIV.jkqWindowContent');

			var template =

				//Get the name for the timer
				'<div class="timer_start timer-toolbar">' +
					'<div class="timername ui-widget-header ui-corner-all">' +
						'<input id="timername" type="text" name="timername" class="ui-corner-all">' +
					'</div>' +
				'</div>' +

				//Write the title for the channel selector
				'<div class="timer_service timer-toolbar">' +
					'<div class="timersvclogoframe ui-widget-header ui-corner-all">' +
						'<div id="timerservicelogo" class="channellogo"></div>' +
					'</div>' +
					'<select id="timerservice" size="1" class="ui-widget-header ui-corner-all">' +
						channelOptions +
					'</select>' +
				'</div>' +

				//Get the timer frequency
				//TODO - Consider changing this to a radio button rather than a drop-down
				'<div class="timer-frequency timer-toolbar">' +
					'<select id="timerfrequency" size="1" class="ui-widget-header ui-corner-all">' +
						'<option value="0">' + tma.i18.frequency[0].name + '</option>' +
						'<option value="1">' + tma.i18.frequency[1].name + '</option>' +
						'<option value="2">' + tma.i18.frequency[2].name + '</option>' +
						'<option value="3">' + tma.i18.frequency[3].name + '</option>' +
						'<option value="4">' + tma.i18.frequency[4].name + '</option>' +
						'<option value="5">' + tma.i18.frequency[5].name + '</option>' +
					'</select>' +

					//Get the type of timer
					'<div id="timertypediv" class="timer_type">' +
						'<input type="radio" id="timertype1" name="timertype" value="1"><label for="timertype1">' + self.i18.type_rec + '</label>' +
						'<input type="radio" id="timertype0" name="timertype" value="0"><label for="timertype0">' + self.i18.type_play + '</label>' +
					'</div>' +
				'</div>' +

				//Get the timer start date
				'<div class="timer_start timer-toolbar">' +
					'<input type="text" id="timerstart" class="datetime">' +
					'<div id="starttoolbar" class="ui-widget-header ui-corner-all"> '+
						'<div id="timerstarticon" class="iconset ic-calendar" title="' + self.i18.select_start + '" />'+
						'<span id="timerstarttext" class="toolbar-value"></span>'+
						'<button id="startless1">-15</button> '+
						'<button id="startless2">-5</button> '+
						'<button id="startless3">-1</button> '+
						'<button id="startmore3">+1</button> '+
						'<button id="startmore2">+5</button> '+
						'<button id="startmore1">+15</button> '+
					'</div> '+
				'</div>' +

				//Get the timer duration
				'<div class="timer_duration timer-toolbar">' +
					'<input type="text" id="timerduration" class="datetime">' +
					'<div id="durtoolbar" class="ui-widget-header ui-corner-all"> '+
						'<div id="timerdurationicon" class="iconset ic-clock" title="' + self.i18.select_duration + '" />'+
						'<span id="timerdurationtext" class="toolbar-value"></span>'+
						'<button id="durless1">-15</button> '+
						'<button id="durless2">-5</button> '+
						'<button id="durless3">-1</button> '+
						'<button id="durmore3">+1</button> '+
						'<button id="durmore2">+5</button> '+
						'<button id="durmore1">+15</button> '+
					'</div>' +
				'</div>' +

				//Get the end time
				'<div class="timer_end timer-toolbar">' +
					'<input type="text" id="timerend" class="datetime">' +
					'<div id="endtoolbar" class="ui-widget-header ui-corner-all"> '+
						'<div id="timerendicon" class="iconset ic-calendar" title="' + self.i18.select_end + '" />'+
						'<span id="timerendtext" class="toolbar-value"></span>'+
						'<button id="endless1">-15</button> '+
						'<button id="endless2">-5</button> '+
						'<button id="endless3">-1</button> '+
						'<button id="endmore3">+1</button> '+
						'<button id="endmore2">+5</button> '+
						'<button id="endmore1">+15</button> '+
					'</div> '+
				'</div>' +

				//Show the buttons and change the name depending upon the function, new or modify.
				'<div class="timer-buttons">' +
					'<input id = "timerbuttonupdate" type="button" value="' + updCaption + '">' +
					//Add a reset button to abort the current changes.
					'<input id = "timerbuttonreset" type="button" value="' + self.i18.button_reset + '" style="margin: 0px 0px 0px 10px;  font-weight: bold;">' +
					'<input id = "timerbuttonabort" type="button" value="' + self.i18.button_abort + '" style="margin: 0px 0px 0px 10px;  font-weight: bold;">' +
				'</div>';

			$('<div id="timeredit" />').append(template).appendTo($content);

			//Start time toolbar
			$('DIV#starttoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerstart", amount, 1 );
			});

			//Duration time toolbar
			$('DIV#durtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerduration", amount, 1 );
			});

			//End time toolbar
			$('DIV#endtoolbar > BUTTON').button({text: true}).click(function() {
				var amount = parseInt( $(this).text() );
				self.adjustTime( "#timerend", amount, 2 );
			});

			editTimerWindow.adjustPos({
				hAlign: 'right',
				effect: 'slide-rtl'
			});

			/**
			 * this could be an alternative time selection
			 * instead of slider access
			 * set controlType: dtpSpinner in dtpOptions
			var dtpSpinner =  {
					create: function(tp_inst, obj, unit, val, min, max, step){
						$('<input class="ui-timepicker-input" value="'+val+'" style="width:50%">')
						.appendTo(obj)
						.spinner({
							min: min,
							max: max,
							step: step,
							change: function(e,ui){ // key events
								// don't call if api was used and not key press
								if(e.originalEvent !== undefined)
									tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							},
							spin: function(e,ui){ // spin events
								tp_inst.control.value(tp_inst, obj, unit, ui.value);
								tp_inst._onTimeChange();
								tp_inst._onSelectHandler();
							}
						});
						return obj;
					},
					options: function(tp_inst, obj, unit, opts, val){
						if(typeof(opts) == 'string' && val !== undefined)
							return obj.find('.ui-timepicker-input').spinner(opts, val);
						return obj.find('.ui-timepicker-input').spinner(opts);
					},
					value: function(tp_inst, obj, unit, val){
						if(val !== undefined)
							return obj.find('.ui-timepicker-input').spinner('value', val);
						return obj.find('.ui-timepicker-input').spinner('value');
					}
			};
			 */

			// setup the datepickers
			var dtpOptions = {
					//controlType: dtpSpinner,
					controlType: 'slider',
					addSliderAccess: true,
					sliderAccessArgs: { touchonly: false },
					firstDay: 1,
					timeFormat: "HH:mm",
					dateFormat: "yy/mm/dd",
					monthNames: [],
					monthNamesShort: [],
					dayNames: [],
					dayNamesShort: [],
					dayNamesMin: [],
					timeText: self.i18.dtp_time_text,
					hourText: self.i18.dtp_hour_text,
					minuteText: self.i18.dtp_minute_text,
					closeText: self.i18.dtp_close_text,
					currentText: self.i18.dtp_current_text,
			};

			$.each(tma.i18.months, function(i, month) {
				dtpOptions.monthNames.push(month.name);
				dtpOptions.monthNamesShort.push(month.shortname);
			});

			$.each(tma.i18.days, function(i, day) {
				dtpOptions.dayNames.push(day.name);
				dtpOptions.dayNamesShort.push(day.shortname);
				dtpOptions.dayNamesMin.push(day.shortname.slice(0, 2));
			});

			//Setup the start date field
			$("#timerstart").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerstart").val( tma.toPvrDate( new Date(selectedDateTime) ) );
					self.refreshTimerEditTimes(1);
				}
			}));

			$("#timerstarticon,#timerstarttext").click(function () {
				//$dateNode.datetimepicker('setDate', new Date( $dateNode.val() ) ); // works, but dtp logs a date/time parsing error
				$("#timerstart").val( function(i, value) {
					return tma.date( 'Y/m/d H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			//Setup the duration field
			$("#timerduration").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerduration").val( tma.toPvrDate( new Date(selectedDateTime) ) );
					self.refreshTimerEditTimes(1);
				}
			}));

			$("#timerdurationicon,#timerdurationtext").click(function () {
				$("#timerduration").val( function(i, value) {
					return tma.date( 'Y/m/d H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			// Setup the end date field
			$("#timerend").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#timerend").val( tma.toPvrDate( new Date(selectedDateTime) ) );
					self.refreshTimerEditTimes(2);
				}
			}));

			$("#timerendicon,#timerendtext").click(function () {
				$("#timerend").val( function(i, value) {
					return tma.date( 'Y/m/d H:i', new Date( value ) );
				} ).datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			$('#timertypediv').buttonset();

			self.resetTimerFields(editTimer);

			// JKIT - update the logo on selection change
			$('#timerservice').change(function() {

				var channel = tma.channel( $(this).val() );
				if (channel) {
					// remove old logo class
					// http://stackoverflow.com/questions/2644299/jquery-removeclass-wildcard
					// and add current
					$('#timerservicelogo').removeClass(function (index, css) {
					    return (css.match (/\blogo-\S+/g) || []).join(' ');
					}).addClass('logo-' + channel.logoid);

				}

			});

			//Make the abort button hide the timer edit window
			$('#timerbuttonabort').button().click(function() {
				editTimerWindow.close();
			});

			//Make the reset button reset the timer to its initial settings
			$('#timerbuttonreset').button().click(function() {
				self.resetTimerFields(editTimer);
			});

			//Make the create/update button do its thing
			$('#timerbuttonupdate').button().click(function(e) {

				e.stopPropagation();

				var tTitle = self.i18.modify_dialog_title ;
				var tText = self.i18.modify_dialog_text;

				if (editTimer.index < 0) {
					tTitle = self.i18.create_dialog_title;
					tText = self.i18.create_dialog_text;
				}

				new jkqDialog({
					title: tTitle,
					text: tText,
					buttons: { ok: tma.i18.respond_yes, jkqDialogCancel: tma.i18.respond_cancel }
				}).confirm(function(button) {

					if (button == 'ok') {

						self.saveTimer(editTimer).done(function() {
							editTimerWindow.close();
						});

					}
				});

			});

		});
		
		return promise.then(function() {
			// send this module instance to following .done()
			return self;
		});

	};

	/**
	 * changing start/duration/end with the toolbar
	 */
	this.adjustTime = function (dateNodeSelector, amount, refreshMode) {

		var $dateNode = $(dateNodeSelector);
		var st = new Date( $dateNode.val() );

		var et = new Date(st.getTime() + (amount * 60000));

		$dateNode.val( tma.toPvrDate(et) );

		this.refreshTimerEditTimes(refreshMode);

	};

	/**
	 * Extract the values from the input fields and reformat them for the API.
	 *
	 * @param {TmaTimer}	editTimer	tma timer object: modify; null: new
	 * @returns {jqXHR}
	 */
	this.saveTimer = function (editTimer) {

		var updateTimer = new TmaTimer();

		jQuery.extend(updateTimer,	editTimer);

		var ttype = $("#timertype1").is(":checked") ? 1 : 0;

		updateTimer.isrec = ttype;

		updateTimer.filename = $("#timername").val() + tma.config.recextension;

		var channel = tma.channel( $("#timerservice").val() );

		updateTimer.servicetype = channel.chtype;
		updateTimer.serviceindex = channel.chnum;

		updateTimer.reservationtype = $("#timerfrequency").val();

		updateTimer.starttime = $("#timerstart").val();
		updateTimer.endtime = $("#timerend").val();

		var td = new Date( $("#timerduration").val() );
		var tdur = (((td.getHours() * 60 ) + td.getMinutes()));

		updateTimer.duration = tdur;

		return tma.apiTimerUpdate(editTimer, updateTimer).done(function(updateObject) {

			if (self.options.onsaved) {
				self.options.onsaved(editTimer);
			}

		}).fail(function(xhr, errorType) {

				new jkqAlert({
					title:	tma.i18.dialog_title_error,
					text:	self.i18.create_modify_error
				}).confirm();

		});

	};

	/**
	 * Initialise the field values for display
	 *
	 * @param {TmaTimer}	editTimer	tma timer object to edit, empty for a new timer
	 */
	this.resetTimerFields = function (editTimer) {

		var channelid = '';

		// set defaults
		if	(typeof editTimer.filename	== 'undefined') editTimer.filename	= this.i18.default_filename + tma.config.recextension;
		if	(typeof editTimer.isrec		== 'undefined') editTimer.isrec		= 1;
		if	(typeof editTimer.logoid	== 'undefined') editTimer.logoid	= $('#timerservice').val();
		  //The default start time is in 5 minutes
		if	(typeof editTimer.starttime	== 'undefined') editTimer.starttime = tma.toPvrDate(new Date(new Date() - tma.timeOffset + 300000));
		//The default duration is 2 hours
		if	(typeof editTimer.duration	== 'undefined') editTimer.duration	= 120;
		if	(typeof editTimer.reservationtype	== 'undefined') editTimer.reservationtype	= 0; // once

		//Set the timer type
		if(editTimer.isrec == 1) {
			$("#timertype1").prop("checked", true);
		} else {
			$("#timertype0").prop("checked", true);
		}

		//Set the name of the timer
		var filename;
		if (tma.config.utftoppy || !editTimer.filename_hex) {
			filename = editTimer.filename;
		} else {
			// JKIT - fallback to filename_hex for non-UFT toppys
			filename = tma.decodeHexname(editTimer.filename_hex);
		}
		$("#timername").val(tma.removeExtension(filename));

		//Set the service
		$('#timerservice').val(editTimer.logoid);
		channelid = editTimer.logoid;

		//Set the timer frequency
		$('#timerfrequency').val(editTimer.reservationtype.toString());

		//Set the start time
		$("#timerstart").val(editTimer.starttime);

		//Set the duration
		var td = new Date("2000/01/01 02:00:00");
		td.setHours(editTimer.duration / 60);
		td.setMinutes(editTimer.duration % 60);
		$("#timerduration").val( tma.toPvrDate(td) );

		//Calculate the end timer
		this.refreshTimerEditTimes(1);

		if (channelid) {
			$('#timerservicelogo').addClass('logo-' + channelid);
		}

	   //Force the UI to update
		$("#timertypediv").buttonset('refresh');

	};

	this.refreshTimerEditTimes = function (refreshMode) {

		var st = new Date( $("#timerstart").val() );
		var td = new Date();
		var et = new Date();

		if ( refreshMode == 1 ) {		//Start + Duration = End

			td = new Date( $("#timerduration").val() );

			var msec = (((td.getHours() * 60 ) + td.getMinutes()) * 60000);
			et = new Date(st.getTime() + msec);

		} else {					//Duration = End - Start

			et = new Date( $("#timerend").val() );

			var diff = (et.getTime() - st.getTime()) / 60000;  //Difference in minutes
			td = new Date("2000/01/01 00:00:00");
			td.setHours(diff / 60);
			td.setMinutes(diff % 60);

		}

		$("#timerstarttext").text(tma.date(this.i18.timerdate_fmt, st));

		$("#timerduration").val( tma.toPvrDate(td) );
		$("#timerdurationtext").text(tma.date("H:i", td));

		$("#timerend").val( tma.toPvrDate(et) );
		$("#timerendtext").text(tma.date(this.i18.timerdate_fmt, et));

	};

}
