/**
 * timers module
 */
tma.registerModule( timerspage );
timerspage.prototype = new tmaModule();
function timerspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timerspage';
	
	this.pageGroup = 'timers';

	// available table columns
	var tableColumns = {
			timertype:	true,
			filename:	true,
			starttime:	true,
			endtime:	true,
			duration:	true,
			channel:	true,
			frequency:	true,
			timerfn:	true
	};

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.parent().setup();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.addPage( this, this.i18.menu_caption );
			
		}

	};

	/**
	 * Display the timers, obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#content').html('<img src="css/img/wait.gif">');

			tma.apiGetTimers().done(function() {

				self.showTimers();

				self.insertConfig();
				
				/** @returns MenuManager */
				var menu;
				if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

					menu.insertItemAfter(
							self.name,
							self,
							'timers-config',
							'<i class="fa fa-gear" />',
							self.toggleConfig
					).addClass('config');
					
				}

			}).fail(function(xhr, e, err) {

				$("#timestamp").text( tma.i18.record_error.format(e, err) );
				// remove pending image
				$('#content').empty();

			});

		});

	};

	/**
	 * refresh the timers, obtained from the PVR in the browser window
	 */
	this.refresh = function() {

		$('#content').html('<img src="css/img/wait.gif">');

		tma.apiGetTimers().done(function() {

			self.showTimers();

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").text( tma.i18.record_error.format(errorType, error) );
			// remove pending image
			$('#content').empty();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// hide and clear config frame, if exists
		$('DIV#config').hide().empty();
		
		/** @returns MenuManager */
		var menu;
		if (menu = tma.fn.runFeature('getMenu', ['MainMenu'])) {

			menu.removeItem('timers-config');
			
		}

	};

	/**
	 * (de)activate config
	 */
	this.toggleConfig = function() {

		$('DIV#config').slideToggle();

	};

	this.showTimers = function () {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.timers.timestamp).toLocaleString() );
		$("#counter"	).text( tma.i18.record_counter.format( tma.timers.count ) );

		// Settings
		var timersFilter = parseInt( this.setting('timersfilter', 0) );
		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var userCols = this.localSetting( 'columns', tableColumns );

		// create Table
		var $table = $('<table/>', {
			'class': "timers jk-list jk-round"
		});

		var head = '<thead><tr>';
		var row  = '<tr>';

		$.each(userCols, function(clmClass) {
			head += '<th class="' + clmClass + '"></th>';
			row  += '<td class="' + clmClass + '"><span/></td>';
		});

		head +=	'</tr></thead>';
		row  +=	'</tr>';

		// create table head
		var $head = $(head).appendTo($table);
		$head.find('TH.timertype'	).text(this.i18.columns.timertype);
		$head.find('TH.filename'	).text(this.i18.columns.filename);
		$head.find('TH.starttime'	).text(this.i18.columns.starttime);
		$head.find('TH.endtime'		).text(this.i18.columns.endtime);
		$head.find('TH.duration'	).text(this.i18.columns.duration);
		$head.find('TH.channel'		).text(this.i18.columns.channel);
		$head.find('TH.frequency'	).text(this.i18.columns.frequency);

		var $tbody = $('<tbody/>').appendTo($table);

		$.each(tma.timers.timers, function() {
			
			/** @returns {TmaTimer} */
			var timer = this;

			$row = $(row).appendTo($tbody);
			$row.prop('timer', timer);
			$row.attr('type', timer.isrec ? 'rec' : 'play');

			$row.find('TD.timertype SPAN'	).text( timer.isrec ? self.i18.type_rec : self.i18.type_play );
			$row.find('TD.filename SPAN'	).text( timer.getName() );
			$row.find('TD.starttime SPAN'	).text( tma.date( self.i18.timerdate_fmt, timer.starttime ) );
			$row.find('TD.endtime SPAN'		).text( tma.date( "H:i", timer.endtime ) );
			$row.find('TD.frequency SPAN'	).text( tma.i18.frequency[timer.reservationtype].name );
			$row.find('TD.duration SPAN'	).text( tma.formatDuration(timer.duration) );

			// channel
			$row.find('TD.channel').html(
					'<div class="channellogo-small logo-small-' + timer.logoid + '" />' +
					'<span class="channelname">' + timer.chname + '</span>'
			);

			$('<button class="jk-but iconset ic-delete" title="' + self.i18.delete_tooltip + '"></button>')
			.appendTo($row.find('TD.timerfn > SPAN'));

			$('<button class="jk-but iconset ic-edit" title="' + self.i18.edit_tooltip + '"></button>')
			.appendTo($row.find('TD.timerfn > SPAN'));

		});

		// filter
		if (timersFilter == 1) $table.find('TR[type="play"]').hide();
		if (timersFilter == 2) $table.find('TR[type="rec"]').hide();

		// hide logo or channel name
		if ( (channelDisplay & 1) == 0 ) $table.find('TD.channel > DIV').hide();
		if ( (channelDisplay & 2) == 0 ) $table.find('TD.channel > SPAN').hide();

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (!display) $table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
		});

		$('DIV#content')
		.append($table)
		.append('<p class="add-timer"><button class="jk-but iconset ic-plus add-timer"><span>' + this.i18.new_tooltip + '</span></button></p>')
		.find('BUTTON.add-timer').click( function(e) {
			// stop event propagation, needed for 'closeOnOutsideClick'
			e.stopPropagation();

			self.editTimer({
				index: -1,	// new timer
			});
		});

		$('DIV#content P.add-timer').width( $('DIV#content TABLE.timers').width() );

		// add event listener to delete timers
		$('TABLE.timers BUTTON.ic-delete').click(function() {

			var $row = $(this).parents('TR');
			var timer = $row.prop('timer');

			var filename;
			if (tma.config.utftoppy || !timer.filename_hex) {
				filename = timer.filename;
			} else {
				filename = tma.decodeHexname(timer.filename_hex);
			}

			new jkqDialog({
				title: self.i18.delete_confirm_caption,
				text:  self.i18.delete_confirm_text.format(tma.removeExtension(filename)),
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'ok') {

					tma.apiDeleteTimer(timer).done(function(delObject) {

						//if (delObject.mpg_status == 0) {

							$row.remove();

						/*} else {

							new jkqAlert({
								title: tma.i18.dialog_title_error,
								text: 'status ' + delObject.mpg_status + ': delete file<br>"' + delObject.file + '"<br>failed!'
							}).confirm();

						}*/

					});

				}

			});

		});

		// DMC add event listener to edit timers
		$('TABLE.timers BUTTON.ic-edit').click(function(e) {

			var timer = $(this).parents('TR').prop('timer');

			self.editTimer(timer);

		});

	};

	this.editTimer = function(timer) {

		if (tma.fn.editTimer) {
			tma.fn.editTimer(
					timer,
					{		// options
						onsaved: function() {
							// reload timers
							tma.apiGetTimers().done(function() {
								self.showTimers();
							});
						}
					}
			);
		}

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({timerfn:	'Buttons'}, this.i18.columns),
				'#content TABLE.timers',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				'#content TABLE.timers',
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
				}
		);

		var $filterSetting = $(
			'<div class="setting">' +
				'<span class="label">' + this.i18.config.filter.caption + '</span>' +
				'<input type="radio" name="timerfilter" id="timerfilter0" value="0"><label for="timerfilter0">' + this.i18.config.filter.all + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter1" value="1"><label for="timerfilter1">' + this.i18.config.filter.rec + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter2" value="2"><label for="timerfilter2">' + this.i18.config.filter.play + '</label> ' +
			'</div>'
		);
		$filterSetting.find('INPUT[name="timerfilter"]').click(function() {

			var timersFilter = $('DIV#config INPUT[name="timerfilter"]:checked').val();
			self.setSetting( 'timersfilter',  timersFilter);

			$('TABLE.timers TR[type]').show();
			if (timersFilter == 1) $('TABLE.timers TR[type="play"]').hide();
			if (timersFilter == 2) $('TABLE.timers TR[type="rec"]').hide();

		}).filter('[value="' +  this.setting('timersfilter', 0) + '"]' ).prop('checked', true);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.append($filterSetting);

	};

}
