function TMSRemote() {

	var self = this;

	//	commands to TMSremote
	this.Cmd_Alive			= 0;
	this.Cmd_Authenticate	= 1;
	this.Cmd_RefreshAll		= 2;
	this.Cmd_KeyEvent		= 3;
	this.Cmd_HideOsd		= 4;
	this.Cmd_Reboot			= 5;
	this.Cmd_VideoFrame		= 6;
	this.Cmd_AsciiEvent		= 7;

	this.KEY_Flag_Push		= 0x02000000;

	//	messages from TMSremote
	this.MID_RecordingDialog	= 0;
	this.MID_AuthFailed			= 1;
	this.MID_DirectOn			= 2;
	this.MID_DirectOff			= 3;
	this.MID_OSDframeComplete	= 4;

	this.directMode = false;
	this.keymap = Array();

	this.keyCodes = {

			code2410: {

				keyOnOff:		0x1000a,
				keyMute:		0x1000c,
				key1:			0x10001,
				key2:			0x10002,
				key3:			0x10003,
				key4:			0x10004,
				key5:			0x10005,
				key6:			0x10006,
				key7:			0x10007,
				key8:			0x10008,
				key9:			0x10009,
				keyRev:			0x1000b,
				key0:			0x10000,
				keyInfo:		0x10014,

				keyVOLup:		0x10047,
				keyVOLdown:		0x10048,
				keyPRGup:		0x10045,
				keyPRGdown:		0x10046,
				keyUp:			0x10012,
				keyDown:		0x1001d,
				keyLeft:		0x1000f,
				keyRight:		0x10015,
				keyOk:			0x1001e,
				keyMenu:		0x1001c,
				keyEpg:			0x10016,
				keyFiles:		0x10042,
				keyExit:		0x10017,

				keyRed:			0x1003f,
				keyGreen:		0x10024,
				keyYellow:		0x10025,
				keyBlue:		0x10026,
				keyWhite:		0x10049,

				keySubtitle:	0x10021,
				keyVideotext:	0x10037,
				keyAudio:		0x10013,
				keyVF:			0x10044,
				keyAR:			0x1000e,
				keyTime:		0x10010,
				keyTvRadio:		0x10018,
				keyRec:			0x1003d,
				keyStop:		0x1003c,
				keySat:			0x10040,
				keyM:			0x10022,
				keyRewind:		0x10038,
				keyPause:		0x1001a,
				keyPlay:		0x10039,
				keyForward:		0x1003a,
				keyPrev:		0x10041,
				keySlow:		0x1003b,
				keyCut:			0x1003e,
				keyNext:		0x10043,

				keyFav:			0x10020,

			},
			code2401: {

				keyOnOff:		0x1000a,
				keyMute:		0x1000c,
				key1:			0x10001,
				key2:			0x10002,
				key3:			0x10003,
				key4:			0x10004,
				key5:			0x10005,
				key6:			0x10006,
				key7:			0x10007,
				key8:			0x10008,
				key9:			0x10009,
				keyRev:			0x1000b,
				key0:			0x10000,
				keyInfo:		0x10014,

				keyVOLup:		0x1003f,
				keyVOLdown:		0x10024,
				keyPRGup:		0x10025,
				keyPRGdown:		0x10026,
				keyUp:			0x10012,
				keyDown:		0x1001d,
				keyLeft:		0x1000f,
				keyRight:		0x10015,
				keyOk:			0x1001e,
				keyMenu:		0x1001c,
				keyEpg:			0x10016,
				keyFiles:		0x10042,
				keyExit:		0x10017,

				keyRed:			0x10047,
				keyGreen:		0x10048,
				keyYellow:		0x10049,
				keyBlue:		0x1004a,
				keyWhite:		0x10049,

				keySubtitle:	0x10021,
				keyVideotext:	0x10037,
				keyAudio:		0x10013,
				keyVF:			0x10045,
				keyAR:			0x1000e,
				keyTime:		0x10010,
				keyTvRadio:		0x10018,
				keyRec:			0x1003d,
				keyStop:		0x1003c,
				keySat:			0x10040,
				keyM:			0x10022,
				keyRewind:		0x10038,
				keyPause:		0x1001a,
				keyPlay:		0x10039,
				keyForward:		0x1003a,
				keyPrev:		0x10041,
				keySlow:		0x1003b,
				keyCut:			0x1003e,
				keyNext:		0x10043,

				keyFav:			0x10020,

			}

	};

	/**
	 * filled on message "OSD frame complete"
	 */
	var socket = null;
	var aliveTimer = null;
	var OsdFrameComplete = false;
	var firstOsdFrameComplete = false;
	var updatedMacroblocks = new Uint8Array(1620);

	/**
	 * log to the console and trigger a log event
	 *
	 * @param msg
	 */
	function log(msg) {

		console.log(msg);
		$(self).triggerHandler('log', [msg]);

	};

	var CRC32Table = null;

	function crc32(str) {

		function create_table() {

			CRC32Table = new Uint32Array(256);
			var poly = 0x04C11DB7;
			var code;

			for (var i = 0; i < 256; i++) {
				code = i << 24;
				for (var j = 0; j < 8; j++) {
					if ((code & 0x80000000) != 0) {
						code = (code << 1) ^ poly;
					} else {
						code = (code << 1);
					}
				}
				CRC32Table[i] = code;
			}

		}

		if (!CRC32Table) create_table();

		var crc = 0xffffffff;

		for (var i = 0; i < str.length; i++) {
			var b = str.charCodeAt(i);
			crc = ( (crc << 8) & 0xffffffff ) ^ CRC32Table[((crc >> 24) ^ b) & 0xff];
		}

		return crc >>> 0; // Uint32

	}

	this.sendCommand = function (cmd, code) {

		if (socket.readyState != WebSocket.OPEN) return;

		var buffer;

		if (typeof code == 'undefined') {
			buffer = new ArrayBuffer(4);
		} else {
			buffer = new ArrayBuffer(8);
		}

		var dv = new DataView(buffer);

		dv.setUint32(0, cmd, true);
		if (buffer.byteLength >= 8) {
			dv.setUint32(4, code, true);
		}

		socket.send(buffer);

	};

	this.sendKey = function ( key ) {

		if (socket.readyState != WebSocket.OPEN) return;

		var buffer = new ArrayBuffer(16);

		var dv = new DataView(buffer);

		dv.setUint32( 0, this.Cmd_KeyEvent, true);
		dv.setUint32( 4, key, true);
		dv.setUint32( 8, this.Cmd_KeyEvent, true);
		dv.setUint32(12, key | this.KEY_Flag_Push, true);

		socket.send(buffer);

	};

	function sendAlive() {

		self.sendCommand(self.Cmd_Alive);

	}

	/**
	 * decode and display a PixelBlock
	 *
	 * @param {CanvasCtx}	ctx		CanvasContext to display the PixelBlock
	 * @param {ArrayBuffer}	buffer	Buffer with the tmsremote block
	 * @returns 1 if blank, 2 if data
	 */
	function decodeBlock(ctx, buffer, blockindex, rleLength) {

		var result = 2;

		var uint8View = new Uint8Array(buffer);

		var pixel = ctx.createImageData(16, 16);

		var index = (blockindex >= 3000) ? blockindex - 3000 : blockindex;
		var isOsd = (blockindex < 1620);

		var x = (index % 45) << 4;
		var y = parseInt(index / 45) << 4;

		var n = 0;
		var i = 8;
		while ( rleLength > 0 ) {

			var blue	= uint8View[i++];
			var green	= uint8View[i++];
			var red		= uint8View[i++];
			var len		= uint8View[i++];

			var opacity = 255;
			// ganz schwarz ist bei OSD transparent
			if ( isOsd && (red == 0) && (green == 0) && (blue == 0) ) {
				opacity = 0;
				if (len == 255) result = 1; // whole block is blank
			}

			for ( var j = 0; j <= len; j++ ) {
				pixel.data[n++] = red;
				pixel.data[n++] = green;
				pixel.data[n++] = blue;
				pixel.data[n++] = opacity;
			}

			rleLength--;

		}

		ctx.putImageData(pixel, x, y);

		return result;

	};

	this.connect = function(host, port, password, osdCtx, tvCtx) {

		var pwHash = crc32(password);

		var url = 'ws://' + host + ':' + port + '/tmsremote';
		try {

			// connect
			//if ( window.WebSocket ) {
			if ( 'WebSocket' in window ) {

				socket = new WebSocket(url, ['binary']);
				log('WebSocket - status ' + socket.readyState);

				socket.binaryType = 'arraybuffer';

				socket.onopen = function(msg) {

					log("Welcome - status " + this.readyState);

					aliveTimer = window.setInterval(function() {
						sendAlive();
					}, 5000);

					OsdFrameComplete = true;
					firstOsdFrameComplete = false;
					self.directMode = false;

					// test: authenticate (with wrong hash)
					//sendCommand(1, 425);
					// authenticate
					self.sendCommand(self.Cmd_Authenticate, pwHash);

					$(self).triggerHandler('ws_open');

				};

				socket.onmessage = function(msg) {

					//log("Received: " + msg.data.byteLength + " Bytes");

					self.decodeMsgData(msg.data, osdCtx, tvCtx);

				};

				socket.onclose = function(msg) {

					$(self).triggerHandler('ws_close');

					log("Disconnected - status " + this.readyState);
					window.clearInterval(aliveTimer);
					socket = null;
					self.directMode = false;

				};

				socket.onerror = function(msg) {
					log("Error: " + msg.data);
				};

			} else {
				log("WebSocket is not available");
			}

		} catch (ex) {
			log(ex);
		}

	};

	this.decodeMsgData = function(block, osdCtx, tvCtx) {

		var dv = new DataView(block);
		var blockIndex = dv.getUint32(0, true);
		var rleLength = dv.getUint32(4, true);

		//log("-block-index: " + blockIndex);
		//log("-rle: " + rleLength);

		if (blockIndex == 2000) {
			// message block
			var msgNr = dv.getUint32(8, true);

			switch (msgNr) {
				case self.MID_AuthFailed:

					log("-Msg: Authentication failed!");
					break;

				case self.MID_DirectOn:

					log("-Msg: Direct Mode On!");
					self.directMode = true;
					break;

				case self.MID_DirectOff:

					log("-Msg: Direct Mode Off!");
					self.directMode = false;
					break;

				case self.MID_OSDframeComplete:

					OsdFrameComplete = true;

					log('-Msg: OSD Frame complete');

					break;

				default:
					log("-Msg: " + msgNr);
				break;
			}

			$(self).triggerHandler('ws_message', [msgNr]);

		} else if ( (blockIndex >= 0) && (blockIndex < 1620) ) {

			if (OsdFrameComplete) {
				OsdFrameComplete = false;
				for ( var i = 0; i < 1620; i++) updatedMacroblocks[i] = 0;
			}

			updatedMacroblocks[blockIndex] = decodeBlock(osdCtx, block, blockIndex, rleLength);

			$(self).triggerHandler('ws_osdblock', [block]);

			if (!firstOsdFrameComplete && (blockIndex == 1619)) {

				firstOsdFrameComplete = true;

				// get initial screenshot
				self.sendCommand(self.Cmd_VideoFrame);

			}

		} else if ( (blockIndex >= 3000) && (blockIndex < 4620) ) {

			decodeBlock(tvCtx, block, blockIndex, rleLength);

		}

	};

	this.quit = function () {

		log("closing...");
		window.clearInterval(aliveTimer);
		socket.close();

	};

	/**
	 * calculate updated region
	 *
	 * @returns {left,top,right,bottom,width,height}
	 */
	this.getUpdatedRegion = function() {

		var ur = {
				left: 44,
				top: 35,
				right: 0,
				bottom: 0
		};
		var i;
		for ( var y = 0; y < 36; y++) {
			for ( var x = 0; x < 45; x++) {
				i = y * 45 + x;
				if (updatedMacroblocks[i]) {
					if (x < ur.left)	ur.left = x;
					if (x > ur.right)	ur.right = x;
					if (y < ur.top)		ur.top = y;
					if (y > ur.bottom)	ur.bottom = y;
				}
			}
		}
		if ( (ur.left > ur.right) || (ur.top > ur.bottom) ) {
			ur.left		= 0;
			ur.width	= 0;
			ur.top		= 0;
			ur.height	= 0;
		} else {
			ur.left		= (ur.left		<< 4);
			ur.top		= (ur.top		<< 4);
			ur.right	= (ur.right		<< 4) + 15;
			ur.bottom	= (ur.bottom	<< 4) + 15;
			ur.width	= ur.right - ur.left + 1;
			ur.height	= ur.bottom - ur.top + 1;
		}

		return ur;

	};

	this.loadKeymap = function() {

		$.get('TMSRemote.ini', function(text) {

			var lines = text.replace( /\r/, '').split('\n');
			var cat = '';
			for ( var i = 0; i < lines.length; i++) {

				var line = lines[i];
				if (
						(line) &&
						(line[0] != ';')
				) {

					var match = line.match(/^\[([^\]]*)\]/);
					if (match) {
						cat = match[1];
					}

					if (cat == 'Keymap') {
						match = line.match(/^([0-9a-f]{2})=([0-9a-f]+)/);
						if (match) {
							self.keymap[ parseInt('0x' + match[1]) ] = parseInt('0x' + match[2]);
						}
					}

				}
			}


		}, 'html').fail(function(xhr, e, err) {

			log("ERROR: " + e + "; " + err);

		});

	};

	$(function() {
		self.loadKeymap();
	});

}

var tmsr = new TMSRemote();
