
/**
 * switch on/off TMSremote.exe like keyboard control
 */
var keyboardControl = true;

var $osdCanvas;
var osdCtx;
var $tvCanvas;
var tvCtx;
var logtext = '';

function log(msg) {
	//$("#log").append(msg + "\n");
	logtext += msg + "\n";
}

$(tmsr).on('log', function (e, msg) {
	log(msg);
});

function create_canvas() {

	$tvCanvas = $('<CANVAS id="tv" width="720" height="576" />').css({
		width: 	$('DIV#tvscreen').width(),
		height: $('DIV#tvscreen').height(),
	}).appendTo('DIV#tvscreen');

	if($tvCanvas.get(0).getContext){

		tvCtx = $tvCanvas.get(0).getContext('2d');

		$osdCanvas = $('<CANVAS id="osd" width="720" height="576" />').css({
			width: 	$('DIV#tvscreen').width(),
			height: $('DIV#tvscreen').height(),
		}).appendTo('DIV#tvscreen');

		osdCtx = $osdCanvas.get(0).getContext('2d');

		return true;

	}

	return false;

}

$(tmsr).on('ws_open', function (e) {

	$('DIV#disconnected').hide();
	$('DIV#connected').show();

	// keyboard control
	//if (keyboardControl)
	$('BODY').on('keypress', function(e) {
		e.preventDefault();
		if (tmsr.directMode) {
			tmsr.sendCommand(tmsr.Cmd_AsciiEvent, e.which);
		} else if (keyboardControl && (tmsr.keymap[e.which]) ) {
			tmsr.sendKey(tmsr.keymap[e.which]);
			log('keypress: ' + e.which + " => " + tmsr.keymap[e.which] + "<br>");
		}
	}).on('keyup', function(e) {
		e.preventDefault();
            //case 9:  // Tab
            //case 13: // Enter
            //case 37: // Left
            //case 38: // Up
            //case 39: // Right
            //case 40: // Down
			//case 46:	// delete
		if (
				(e.keyCode == 8) &&
				(tmsr.directMode)
		) {
			// backspace
			tmsr.sendCommand(tmsr.Cmd_AsciiEvent, e.keyCode);
		} else if (keyboardControl && !tmsr.directMode) {

			if (tmsr.keymap[e.keyCode]) {
				tmsr.sendKey(tmsr.keymap[e.keyCode]);
				log('keyup: ' + e.keyCode + " => " + tmsr.keymap[e.keyCode] + "<br>");
			}

		}
	});

});

$(tmsr).on('ws_close', function (e) {

	$('DIV#disconnected').show();
	$('DIV#connected').hide();
	// direct mod off
	$('SPAN.direct-mode').attr('mode', 'off');

	$('BODY').off('keypress').off('keyup');

	$('CANVAS').remove();

});

$(tmsr).on('ws_message', function (e, msgNr) {

	switch (msgNr) {

		case tmsr.MID_DirectOn:

			$('SPAN.direct-mode').attr('mode', 'on');
			break;

		case tmsr.MID_DirectOff:

			$('SPAN.direct-mode').attr('mode', 'off');
			break;

	}

});

function setKeyCodes(codeTable) {

	var code = tmsr.keyCodes[codeTable];

	$.each(code, function(key, code) {

		$('.remote BUTTON.' + key).attr('keyCode', code);

	});

}

function setRemotePos() {

	var offs = $('DIV#tvscreen').offset();
	$('DIV.remote').css({
		top: offs.top,
		left: offs.left + parseInt($('DIV#tvscreen').width()) + 20
	});

}

function setTvSize(sizeStr) {

	var size = sizeStr.split('x');
	if (size.length == 2) {

		$('DIV#tvscreen').css({
			width: 	parseInt(size[0]),
			height: parseInt(size[1]),
		});

		$('DIV#tvscreen CANVAS').css({
			width: 	parseInt(size[0]),
			height: parseInt(size[1]),
		});

		setRemotePos();

	}

}

$(function() {

	setRemotePos();

	// connect
	$('BUTTON#connect').click(function() {

		$('INPUT#hideOsd').attr('checked', false);

		if (create_canvas()) {

			var host		= $('INPUT[name="host"]').val();
			var port		= $('INPUT[name="port"]').val();
			var password	= $('INPUT[name="password"]').val();

			tmsr.connect(host, port, password, osdCtx, tvCtx);

		} else {

			alert('Error: canvas feature not available!');

		}

	});

	// disconnect
	$('BUTTON#quit').click(function() {
		$('BODY').off('keypress').off('keyup');
		tmsr.quit();
	});

	// remote buttons
	$('DIV.remote BUTTON').click(function() {

		if ( $('DIV#connected').is(':visible') ) {
			var keyCode = parseInt($(this).attr('keyCode'));

			tmsr.sendKey(keyCode);
		}

	});

	// remote selection
	$('SELECT[name="remoteTypes"]').on('change', function() {

		var type = $(this).val();
		$('DIV.remote').hide().filter('.' + type ).show();

	}).val('srp2410').change();

	// remote code selection
	$('SELECT[name="remoteCodes"]').on('change', function() {

		setKeyCodes( $(this).val() );

	}).val('code2410').change();

	// refresh TV screen image
	$('BUTTON#refreshTv').click(function() {

		tmsr.sendCommand(tmsr.Cmd_VideoFrame);

	});

	// screen size selection
	$('SELECT[name="viewSize"]').on('change', function() {

		setTvSize( $(this).val() );

	});

	// toggle OSD visibility on toppy
	$('INPUT#hideOsd').click(function() {

		var value = $(this).is(':checked') ? 1 : 0;

		tmsr.sendCommand(tmsr.Cmd_HideOsd, value);

	});

	// refresh log
	$("#refreshlog").click(function() {
		$("#log").val(logtext);
		logtext = '';
	});

});

